//============================
// C H A R T   D A   V I N C I 
// ===========================
//
// Author: Keith Howard
// Company: Actuate Corp. San Mateo, CA.
// Release: 1
// Summary: A JavaScript function to enable easy programmatic configuration of BIRT charts.
// Purpose: 
// i) Enables simplified programmatic setting of common chart characteristics.
// ii) Provides aesthetically enhanced charts. (See documentation for features.)
// iii) Provides for easy implementation to support 
// a common look and feel of multiple charts across an entire suite of reports.
// Chart Da Vinci removes the error prone chore of configuring each chart separately 
// to ensure all chart looks the same in every report.
// iv) Provides for highly simplified chart building. Only the chart data needs configuring.
// A default collection of aesthetically enhanced features are applied to the chart.
// v) Tested for:
// -  Bar charts (2D and 2D with Depth) both Stacked and Side-By-Side series.  
// -  Area charts (2D and 2D with Depth) both Stacked and Side-By-Side series. 
// -  Pie charts (2D and 2D with Depth) 
// -  Line charts with single left side axis.   
// vi) Provides a working case study and an excellent source of code samples
// for developers wishing to add scripting to BIRT charts.
// 
//*************************************************************

importPackage(Packages.org.eclipse.birt.chart.model.attribute);
importPackage(Packages.org.eclipse.birt.chart.model.attribute.impl); 
importPackage(Packages.org.eclipse.birt.chart.model.component.impl); 
importPackage(Packages.org.eclipse.birt.chart.model.data.impl); 

// ======================================================
// C O M M O N   V A R I A B L E S   used throughout cdv 
// ======================================================
yax	= "";			// Y AXIS
yax2 = "";			// 2nd Y Axis on right side.
xaxSeries = ""		// Series Definitions (X AXIS)
yaxSeries = ""		// Y Axis Series
ySeriesSize = "";	// Number of Y AXIS series.

chartMode = "";		// "WITH" or "WITHOUT" AXES
chartType = "";		// Chart Type Pie,Bar,Area etc.
chartDim = "";		// Chart Dimension 2D, 2d with DEPTH, 3D

// Y AXIS origins and value
var pyaxOrigin	= null;
var pyaxValue	= null;

// LINE COLORS: MAIN and DARKER
var pLineColorM = "";
var pLineColorD = "";

//=====================================================================
// C O L O R   S H A D E S  and  C O L O R   S C H E M E S
// Create arrays to hold colors, and initialize the entries with colors
//=====================================================================

// COLOR SCHEME
// ============
var pScheme	= "";	// Holds the default color scheme

// Array of Colors - Default colors
// ================================
bjHeather = 	new Array();
bjEmerald = 	new Array();
bjYellow = 		new Array();
bjBlue = 		new Array();
bjRed = 		new Array();
bjRose = 		new Array();
bjNutmeg	=	new Array();
bjMauve		=	new Array();
bjPostit	=	new Array();
bjCharcol	=	new Array();
bjOrange	=	new Array();
bjTeal		=	new Array();

// BJ Collections 1-3, 1 = Lightest Colors,  2 = Medium, 3 = Darkest Colors
bj1 = 			new Array();
bj2 = 			new Array();
bj3 = 			new Array();
	
//*************************************************************
// C H A R T   D A   V I N C I  : Main entry point
//*************************************************************

function cdvBuildChart(chart, icsc, styling1, styling2, styling3)
{

// ==============================
// DISCOVER CHART CHARACTERISTICS
// ==============================

// DISCOVER CHART TYPE (eg: Pie Chart, Area Chart, Line Chart, Bar Chart)
// ======================================================================
chartType = chart.getType();

// DISCOVER CHART SUB-TYPE
// =======================
chartSubType = chart.getSubType();   // eg Side-by-Side, Stacked, Standard

// DISCOVER CHART DIMENSION (2D, 2D with Depth, or 3D)
// ===================================================
var ChartDimension = chart.getDimension().getValue();
if (ChartDimension == chart.getDimension().TWO_DIMENSIONAL_WITH_DEPTH)	{ chartDim = "2D_DEPTH"; } 
if (ChartDimension == chart.getDimension().TWO_DIMENSIONAL)				{ chartDim = "2D_PLAIN"; } 
if (ChartDimension == chart.getDimension().THREE_DIMENSIONAL)			{ chartDim = "3D"; } 

// DISCOVER CHART CLASS (Either: ChartWithoutAxesImpl or ChartWithAxesImpl) 
// ======================================================================== 
var ctype = chart.getClass().toString();
if(  ( ctype.indexOf("ChartWithAxesImpl") )    != -1) { chartMode = "WITHAXES";   };	// Bar,Line,Area
if(  ( ctype.indexOf("ChartWithoutAxesImpl") ) != -1) { chartMode = "WITHOUTAXES"; };	// Pie

// ====================================================
// D E F A U L T   C O L O R    S E T T I N G S
// ====================================================

// INITIALIZE COLOR ARRAYS 
// =======================
cdvSetColorArrays();

// SET DEFAULT PALETTE
// ===================
pScheme = bj3;	// Set default palette of colors

// DEFAULT BAR COLOR (Single series)
// ================================
pBarColor = cdvCopyInstance(bjBlue[2]);

// CHART AREA COLORS
// =================
p01ChartAreaColor		= ColorDefinitionImpl.WHITE();
p02AreaInclAxesColor	= ColorDefinitionImpl.TRANSPARENT();
p03AreaWithinAxesColor  = ColorDefinitionImpl.TRANSPARENT();  // Back wall

// LIGHT/DARK STRIPES and BG WALL Back CONTRAST COLORS
// =====================================================
pLighterColor	= ColorDefinitionImpl.create(248,248,248); // 
pDarkerColor	= ColorDefinitionImpl.create(229,232,224); // 
pFloorColor		= ColorDefinitionImpl.create(221,221,221); //
pSideWallColor	= cdvCopyInstance(pDarkerColor);

// COLOR BY - DEFAULT
// ==================
// By Categories ("C") is usually for single series bar/area charts.
// Each bar is a different color, or for pie charts; each segment is a different color.
pColorBy = "C";	// C = by Categories is Default

// CUSTOM PALETTE - if provided
//=============================
if (styling3 != null)  // IF 390
{

customColors = new Array();
for (p=0; p < (styling3.length) ; p++)
	{ customColors[p]  = cdvRGBtoColorDefinitionImpl(styling3[p]); } 		
pScheme = customColors;	// Set custom colors as the palette colors
} // styling3 END IF 390

// AXIS TITLES - DEFAULT FONT SETTINGS (X and Y AXIS)
// =================================================
axisTitleFontColor = ColorDefinitionImpl.create( 60,60,60 );

// =====================
// DEFAULT LINE COLORS
// =====================
pLineColorM = ColorDefinitionImpl.create(100,100,100); // Default line color main
pLineColorD = ColorDefinitionImpl.create( 70, 70, 70); // Default line color darker

// Default X and Y Grid Line colors 
// --------------------------------
pGridLineColorX = cdvCopyInstance(pLineColorM);
pGridLineColorY = cdvCopyInstance(pLineColorM);

// pLineColorD is a darker shade of the grid line 
// It is used for:
// 1) Client Area outline
// 2) Y and X Axis lines 
// 3) Grid lines 

// DEAFULT BAR OUTLINE COLOR - Based on Chart Dimension 
// ====================================================
if (chartDim == "2D_DEPTH" ) 
pBarOutLine	= ColorDefinitionImpl.TRANSPARENT();	// Used to outline bars 2D with Depth bar charts
else
pBarOutLine	= cdvCopyInstance(pLineColorD);			// Used to outline bars 2D plain bar charts

// ====================================
// CHART TITLE - Default Settings
// ====================================
chart.getTitle().getLabel().getCaption().getFont().setSize(14);
chart.getTitle().getLabel().getCaption().getFont().unsetBold();
chart.getTitle().getLabel().getCaption().getFont().setName("Arial");	
chart.getTitle().setVisible(true);


// SERIES LABELS - DEFAULT
// =======================
pSeriesLabelsShow = false;		
pSeriesLabelsFontFace = "Serif"
pSeriesLabelsFontSize = 9;
pSeriesLabelsFontColor = ColorDefinitionImpl.BLACK();
pSeriesLabelsBGColor =   ColorDefinitionImpl.TRANSPARENT();
pSeriesLabelsBold =   false;
pSeriesLabelsItalic =   false;
pSeriesLabelsUnderline =   false;
pSeriesLabelsPosition =   "outside";
pSeriesLabelsRotation = 0;

// LINE CHART STYLING  - DEFAULTS
// ==============================
pLineThickness = 1;
pLineMarkerShape = "box";
pLineMarkerSize	= 1;
pLineMarkerColor = ColorDefinitionImpl.BLACK();

// MULTILINE X AXIS LABELS
// =======================
pML = null;	// Default is single line X axis labels. Options are: "left" or "centered"

// DEFAULT AREA separator and STRIPE Outline color
// =====================================
pStripeOutline	= "NONE";	// "NONE" = No Stripe Outline 
pAreaSeparator	= ColorDefinitionImpl.TRANSPARENT(); // Used to separete areas on area charts


// ALL CHARTS: 2D with DEPTH : Depth setting
// =========================================
if (chartDim == "2D_DEPTH" ) // IF 717
{
// =============================
// CHART DEPTH (2D with depth only)
// =============================
// Used to alter the tilt angle of 2D with axes charts. Default = 10.
// Default = 10.  0-9 = Lesser angle, tending to flat.
// 11+ makes the angle stronger.

pDepthAngle = 5;		// cdv default depth (aka: series thickness)

if (  (styling1 != null) && (styling1[15]!= null)	 )
{ 
	pDepthAngle = parseFloat(styling1[15]); 
	chart.setSeriesThickness(pDepthAngle);
} 

// Set cdv default depth to 5 or user setting.
// BIRT default is 10.
chart.setSeriesThickness(pDepthAngle);

} // END-IF 717


// IF CHART WITHAXES
// 1) DISCOVER CHART ORIENTATION (Vertical -or- Horizontal)
// 2) GET PRIMARY AXIS (X & Y)
// ====================================================
if (chartMode == "WITHAXES")	//IF -103a Only for charts with axes
{
	chartOrientation = chart.getOrientation();	// Vertical -or- Horizontal
	
// GET PRIMARY AXIS, X and Y
// =========================
	xax = chart.getBaseAxes()[0];	
	// yax = chart.getPrimaryOrthogonalAxis(xax);  // Alt way to get main Y-Axis
	//
	// Get Associated Y axes for this X axis
	// Usually there is only 1 Y Axis. 
	// Sometimes 2 with multi-series charts. Another Y Axis on the right side.
	//
	assocAxes = xax.getAssociatedAxes();	
	if (assocAxes.size() == 1 )
	{
		yax		= assocAxes.get(0);  // Get the 1 Y Axis
		yax2 	= null;
	}
	else
	{
		// Get both, usually 2, Y Axis
		yax		= assocAxes.get(0);
		yax2	= assocAxes.get(1);
	}

// GET SERIES DEFINITIONS - X AXIS
// ================================
	xaxSeries		= xax.getSeriesDefinitions().get(0);
	
// GET SERIES DEFINITIONS - Y AXIS - 1st Y AXIS
// ============================================
	ysdefs		= yax.getSeriesDefinitions().get(0);
	yaxSeries	= ysdefs.getSeries();
	ySeriesSize	= ysdefs.getSeries().size();
	

// X AXIS TITLE
// ============
xax.getTitle().setVisible(true);
xax.getTitle().getCaption().getFont().setName("San Serif");	
xax.getTitle().getCaption().getFont().setSize(12);
xax.getTitle().getCaption().setColor( cdvCopyInstance( axisTitleFontColor ) );  

// Adjust insets on X Axis title
xax.getTitle().getInsets().setTop(3.0);		// Leave gap between title and labels
xax.getTitle().getInsets().setBottom(4.0);	// Leave gap between title and legend


// Y AXIS TITLE
// ============
yax.getTitle().setVisible(true);
yax.getTitle().getCaption().getFont().setName("San Serif");	
yax.getTitle().getCaption().getFont().setSize(12);	
yax.getTitle().getCaption().setColor( cdvCopyInstance( axisTitleFontColor ) );
yax.getTitle().getInsets().setBottom(9.0);  // Leave gap between title and labels


// SET DEFAULT INSETs for AXIS TITLEs. Both X and Y AXES	// 104
// ===============================================================
if(chartOrientation == "Vertical")
{
xax.getTitle().getInsets().setTop(3.0);		// Leave gap between title and labels
xax.getTitle().getInsets().setBottom(4.0);	// Leave gap between title and legend
yax.getTitle().getInsets().setBottom(9.0);  // Y AXIS - Leave gap between title and labels
}
else
{
// Horizontal chart
yax.getTitle().getInsets().setTop(3.0);		// Leave gap between title and labels
yax.getTitle().getInsets().setBottom(4.0);	// Leave gap between title and legend
xax.getTitle().getInsets().setBottom(9.0);  // X AXIS - Leave gap between title and labels
}

// SET X AXIS LABELS - DEFAULT FONT SETTINGS
// =========================================	
xax.getLabel().getCaption().getFont().setName("San Serif");	
xax.getLabel().getCaption().getFont().setSize(8);	
xax.getLabel().setVisible(true);


// SET Y AXIS LABELS - DEFAULT FONT SETTINGS
// =========================================
yax.getLabel().getCaption().getFont().setName("San Serif");	
yax.getLabel().getCaption().getFont().setSize(8);	
yax.getLabel().setVisible(true);

if( yax2 != null)
{
yax2.getLabel().getCaption().getFont().setName("San Serif");	
yax2.getLabel().getCaption().getFont().setSize(8);	
yax2.getLabel().setVisible(true);
}

// Y AXIS GRID LINES - DEFAULTS 
// (Do not confuse with AXIS lines)
// ================================
pGridLineShowY = "show";
pGridLineWidthY	= 0;		// Width of grid line on back wall
pGridLineStyleY	= "solid"	// Grid line style default
pGridLineHideTop = "n";		// Hide top grid line (Default = "n" no)

// X AXIS GRID LINES  - DEFAULTS
// =============================
pGridLineShowX = "hide";
pGridLineWidthX	= 0;		// Width of grid line on back wall
pGridLineStyleX	= "solid";	// Gridline style default


// SPACING UNITS on X AXIS IF 309
// ==============================
// ONLY: Charts With Axes.
// Adjust the gap (spacing) between groups of bars, effectively making the bars fatter or thinner.
// Useful with bar charts with multiple series, which are clustered as a group of bars. 
// This setting will make bars fatter or thinner by altering the gap/sapce between the groups of bar.
// Very useful if data point labels are being displayed above each bar.
// By making the bars fatter, the label will have room to fit without
// over running the adjacent bar.
//
// Default = 50
// 1-49 = Make bars fatter (by reducing the intergroup space)
// 51+ = Make bares thinner ((by increasing the intergroup space)
// =========================================

if (  (styling1 != null) && (styling1[16]!= null)	 )
{ 
	pSpacingUnits = parseInt(styling1[16]);
	chart.setUnitSpacing(pSpacingUnits);
} // END-IF 309


// TICK MARK DEFAULTS
// ==================
pTickMarkShowX = "show";
pTickMarkShowY = "show";

pTickMarkColorDefault = ColorDefinitionImpl.create(196,196,196);
pTickMarkColorX = cdvCopyInstance(pTickMarkColorDefault);
pTickMarkColorY = cdvCopyInstance(pTickMarkColorDefault);

// X TICK MARKS
pTickMarkPositionX = "below";
pTickMarkWidthX = 1;
pTickMarkStyleX = "solid";

// Y TICK MARKS (Left side axis) 
pTickMarkPositionY = "left";
pTickMarkWidthY = 1;
pTickMarkStyleY = "solid";

// Y TICK MARKS (Right side axis) 
pTickMarkPositionYRight = "right";
pTickMarkWidthYRight = 1;
pTickMarkStyleYRight = "solid";

} // END IF-103a - WITHAXES

// STRIPES DEFAULT = YES.
// ======================
pPaintStripes 	= "Y";	// No stripes


// ========================================
// C U S T O M   U S E R   S T Y L I N G  
// ========================================

// =================================
// STYLING1 - Color choices
// =================================

if (styling1 != null)	// IF Styling 1 (Did user submit sytyling1 choices?)
{	

// =================================
// Color BY CHOICE
// =================================
if (styling1[1] != null) 
{	pColorBy = styling1[1]; // C or S Color Bar by
}

// =================================
// Color B Y   C A T E G O R Y
// =================================
if (styling1[2] != null) 	// IF LMD
{
    // SET PALETTE Colors for By Category 

var pShadeChoice = styling1[2].toLowerCase();
switch (pShadeChoice)
{
	case "light":	{ pScheme = bj1; };    break;
	case "medium":	{ pScheme = bj2; };    break;
	case "dark":	{ pScheme = bj3; };    break;
    	default:	{ pScheme = bj3; };    break;
} // End switch

} // END-IF LMD


// =================
// CHART AREA COLORS
// =================
if (styling1[20]!= null) 	{ p01ChartAreaColor			= cdvRGBtoColorDefinitionImpl(styling1[20]); }
if (styling1[30]!= null) 	{ p02AreaInclAxesColor		= cdvRGBtoColorDefinitionImpl(styling1[30]); }
if (styling1[40]!= null) 	{ p03AreaWithinAxesColor	= cdvRGBtoColorDefinitionImpl(styling1[40]); }


if (chartMode == "WITHAXES")   {  // IF-108a

// =========================================
// WALL and FLOOR COLORS for WITHAXES CHARTS
// ==========================================
if (styling1[41] != null) { pSideWallColor	= cdvRGBtoColorDefinitionImpl(styling1[41]); }
if (styling1[42] != null) { pFloorColor		= cdvRGBtoColorDefinitionImpl(styling1[42]); }

// =============================================
// SINGLE SERIES BAR COLOR
// Custom bar color - Single color for all bars
// ==================================================
if(styling1[43] != null) pBarColor   =  cdvRGBtoColorDefinitionImpl(styling1[43]); // End - styling1[43]

// ===========
// BAR OUTLINE 
// ===========
if (styling1[44] != null) 
{ 
	if (styling1[44] == "none")	// No bar outline
		pBarOutLine	= ColorDefinitionImpl.TRANSPARENT();
	else 
		pBarOutLine	= cdvRGBtoColorDefinitionImpl(styling1[44]); 
}

// ==================
// LINE COLORS
// ==================

// OVERRIDE LINE COLOR - IF USER OVERRIDE
if (styling1[4] != null) 
{ 
	pLineColorM = cdvRGBtoColorDefinitionImpl(styling1[4]); 
	pGridLineColorX = cdvCopyInstance(pLineColorM);
	pGridLineColorY = cdvCopyInstance(pLineColorM);
}

if (styling1[5] != null) 
{ 
	pLineColorD = cdvRGBtoColorDefinitionImpl(styling1[5]);
}

// ===========
// STRIPES
// ===========
if (styling1[8] != null)	{ pPaintStripes	= styling1[8]; }  // Stripes wanted? Y or N
if (styling1[9] != null)  	{ pLighterColor	= cdvRGBtoColorDefinitionImpl(styling1[9]); }
if (styling1[10]!= null) 	{ pDarkerColor	= cdvRGBtoColorDefinitionImpl(styling1[10]); }
if (styling1[11]!= null) 	{ pStripeOutline = styling1[11].toLowerCase(); }
if (styling1[12]!= null) 	{ pAreaSeparator = cdvRGBtoColorDefinitionImpl(styling1[12]); }  // Area separator color

// =============
// GRID LINES
// =============

// X GRID LINES (Usually vertical. Default = hide)
// -----------------------------------------------
if (styling1[50] != null) { pGridLineShowX	= styling1[50].toLowerCase(); } // show/hide. Default = hide 
if (styling1[51] != null) { pGridLineWidthX	= parseFloat(styling1[51]); } // WIDTH of X GRID LINE on BACK WALL
if (styling1[52] != null) { pGridLineColorX	= cdvRGBtoColorDefinitionImpl(styling1[52])  } // COLOR of X GRID LINE on BACK WALL
if (styling1[53] != null) { pGridLineStyleX	= styling1[53].toLowerCase();	}

// X Axis Origins and Values
// -------------------------
// Origin/Intersection Type:  max, min or value
// If value, set the actual value, if submitted by user

if (styling1 != null) { 
if (styling1[57] != null) // IF 1[57]	
{
	var xaxOrigin = styling1[57].toLowerCase();

	switch ( xaxOrigin )
	{
	case "min":	{ xax.getOrigin().setType(IntersectionType.MIN_LITERAL ) };    break;
	case "max":	{ xax.getOrigin().setType(IntersectionType.MAX_LITERAL ) };    break;
	case "value":	{ xax.getOrigin().setType(IntersectionType.VALUE_LITERAL ) };  break;
	} // End switch
} // END IF 1[57]	

if  ( (styling1[58] != null) && (xaxOrigin == "value") )
{
	xaxOriginValue = NumberDataElementImpl.create( parseFloat(styling1[58]) );
	xax.getOrigin().setValue(xaxOriginValue);
}
} // END-IF ORIGINS X


// Y GRID LINES (Usually horizontal. Default = show)
// -------------------------------------------------
if (styling1[60] != null) { pGridLineShowY	= styling1[60].toLowerCase(); } // show/hide. Default = show 
if (styling1[61] != null) { pGridLineWidthY	= parseFloat(styling1[61]); } // WIDTH of Y GRID LINE on BACK WALL
if (styling1[62] != null) { pGridLineColorY	= cdvRGBtoColorDefinitionImpl(styling1[62]) } // COLOR of Y GRID LINE on BACK WALL
if (styling1[63] != null) { pGridLineStyleY	= styling1[63].toLowerCase();	}
// Hide top line
if (styling1[69] != null) { pGridLineHideTop	= styling1[69].toLowerCase();	}

// Y Axis Origins and Values
// -------------------------

// Origin/Intersection Type:  max, min or value
// If value, set the actual value, if submitted by user

if (styling1 != null) { 
if (styling1[67] != null) // IF 1[67]	
{
	var yaxOrigin = styling1[67].toLowerCase();

	switch ( yaxOrigin )
	{
	case "min":	{ yax.getOrigin().setType(IntersectionType.MIN_LITERAL ) };    break;
	case "max":	{ yax.getOrigin().setType(IntersectionType.MAX_LITERAL ) };    break;
	case "value":	{ yax.getOrigin().setType(IntersectionType.VALUE_LITERAL ) };  break;
	} // End switch
} // END IF 1[67]	

if  ( (styling1[68] != null) && (yaxOrigin == "value") )
{
	yaxOriginValue = NumberDataElementImpl.create( parseFloat(styling1[68]) );
	yax.getOrigin().setValue(yaxOriginValue);
}
} // END-IF ORIGINS


// =============================
// X-AXIS TICK MARKS 
// =============================
if (styling1[70] != null) { pTickMarkShowX	= styling1[70].toLowerCase(); } // show/hide. Default = show
if (styling1[71] != null) { pTickMarkWidthX	= parseFloat(styling1[71]); } // WIDTH of X TICK
if (styling1[72] != null) { pTickMarkColorX	= cdvRGBtoColorDefinitionImpl(styling1[72]); } // COLOR of X TICK
if (styling1[73] != null) { pTickMarkStyleX	= styling1[73].toLowerCase();	}
if (styling1[74] != null) { pTickMarkPositionX	= styling1[74].toLowerCase();	}  // ABOVE BELOW etc..

// =============================
// Y-AXIS TICK MARKS 
// =============================
if (styling1[80] != null) { pTickMarkShowY	= styling1[80].toLowerCase(); } // show/hide. Default = hide 
if (styling1[81] != null) { pTickMarkWidthY	= parseFloat(styling1[81]); } // WIDTH of Y TICK
if (styling1[82] != null) { pTickMarkColorY	= cdvRGBtoColorDefinitionImpl(styling1[82]); } // COLOR of Y TICK
if (styling1[83] != null) { pTickMarkStyleY	= styling1[83].toLowerCase();	}
if (styling1[84] != null) { pTickMarkPositionY	= styling1[84].toLowerCase();	}  // ABOVE BELOW etc..

// LINE CHART - MARKERS and LINE THICKNESS
// =======================================
if ( styling1[91] != null) { pLineThickness = parseInt(styling1[91]); }
if ( styling1[92] != null) { pLineMarkerShape = styling1[92].toLowerCase(); }
if ( styling1[93] != null) { pLineMarkerSize = parseInt(styling1[93]); }
if ( styling1[94] != null) { pLineMarkerColor = cdvRGBtoColorDefinitionImpl(styling1[94]);  }

}  // END IF-108a   End of chartMode == "WITHAXES"
}  // END stlying1 provided


// ===========================================================
// STYLING2 - TEXT for CHARTS (Labels, Series ID, Title etc..)
// ===========================================================
if (styling2 != null)
{
// Here if styling2 is defined

// ============
// CHART TITLE 
// ============
if (  (styling2[00]!= null) &&  (styling2[00].toLowerCase() == "hide"))  
	{ chart.getTitle().setVisible(false);  } 	// Hide the title

else  // ELSE 406

{
// CHART TITLE STYLING
// ===================
titleCaption = chart.getTitle().getLabel().getCaption(); // Get Title caption

if (styling2[01]!= null) { titleCaption.setValue(styling2[1]); }
if (styling2[02]!= null) { titleCaption.getFont().setName(styling2[02]) } // Font Face
if (styling2[03]!= null) { titleCaption.getFont().setSize(parseInt(styling2[03]) ) } // Font Size
if (styling2[04]!= null) { titleCaption.setColor( cdvRGBtoColorDefinitionImpl(styling2[04])  ) } // Font Color
if (styling2[05]!= null) { chart.getTitle().setBackground( cdvRGBtoColorDefinitionImpl(styling2[05])  ) } 

if (styling2[06]!= null) // IF 407
{
	fontstyle = styling2[06].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )	{ titleCaption.getFont().setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )	{ titleCaption.getFont().setItalic(true); }
} // END IF 407


// TITLE PLACEMENT (ANCHOR)
// ========================
if (styling2[07]!= null) // IF anchor7
{
titleAnchorPlace = chart.getTitle();
titleAnchor = styling2[07].toLowerCase();

if ( titleAnchor == "e") titleAnchorPlace.setAnchor(Anchor.EAST_LITERAL);  
if ( titleAnchor == "w") titleAnchorPlace.setAnchor(Anchor.WEST_LITERAL);  
if ( titleAnchor == "n") titleAnchorPlace.setAnchor(Anchor.NORTH_LITERAL);  
if ( titleAnchor == "s") titleAnchorPlace.setAnchor(Anchor.SOUTH_LITERAL);  

if ( titleAnchor == "sw") titleAnchorPlace.setAnchor(Anchor.SOUTH_WEST_LITERAL );  
if ( titleAnchor == "se") titleAnchorPlace.setAnchor(Anchor.SOUTH_EAST_LITERAL );  
if ( titleAnchor == "nw") titleAnchorPlace.setAnchor(Anchor.NORTH_WEST_LITERAL );  
if ( titleAnchor == "ne") titleAnchorPlace.setAnchor(Anchor.NORTH_EAST_LITERAL );  
	
} // END-IF anchor7


if (styling2[08]!= null) // IF Horizontal Alignment 818
{
// Get Font alignment object
fontAlign = chart.getTitle().getLabel().getCaption().getFont().getAlignment();
fontA = styling2[8].toLowerCase();
switch (fontA)
{
  case "left": fontAlign.setHorizontalAlignment(HorizontalAlignment.LEFT_LITERAL); break;
  case "center": fontAlign.setHorizontalAlignment(HorizontalAlignment.CENTER_LITERAL); break;
  case "right": fontAlign.setHorizontalAlignment(HorizontalAlignment.RIGHT_LITERAL); break;
  default: fontAlign.setHorizontalAlignment(HorizontalAlignment.CENTER_LITERAL); break;
}
} // END 818

// TITLE INSETS  : Insets - Fine tuning
// ====================================    
if(styling2[09] != null)  // IF-t09
{
	insetsTitle = chart.getTitle().getInsets();
	applyInsets(styling2[09], insetsTitle , icsc);  
}  // END IF-t09		    	
} // END ELSE 406 - End chart title styling


if (chartMode == "WITHAXES")   {  // IF-103b Start

// A X I S    T I T L E S
// ======================

// SET X-AXIS TITLE
// ---------------- 
if (  (styling2[10]!= null) &&  (styling2[10].toLowerCase() == "hide")   )   
	{  xax.getTitle().setVisible(false);  } 

if (xax.getTitle().isVisible()  )  // 210
{   
XAxisTitleCaption = xax.getTitle().getCaption() // Get caption of X AXIS TITLE

if (styling2[11]!= null) { XAxisTitleCaption.setValue(styling2[11]);  } // Title Text
if (styling2[12]!= null) { XAxisTitleCaption.getFont().setName(styling2[12]) } // Font Face
if (styling2[13]!= null) { XAxisTitleCaption.getFont().setSize(parseInt(styling2[13]) ) } //  Font Size
if (styling2[14]!= null) { XAxisTitleCaption.setColor( cdvRGBtoColorDefinitionImpl(styling2[14])  ) } // Font Color
if (styling2[15]!= null) { xax.getTitle().setBackground( cdvRGBtoColorDefinitionImpl(styling2[15])  ) } // BG color

if (styling2[16] != null) 
{
	fontstyle = styling2[16].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )	{ XAxisTitleCaption.getFont().setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )	{ XAxisTitleCaption.getFont().setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 )	{ XAxisTitleCaption.getFont().setUnderline(true); }
}

} // END IF 210

// SET Y-AXIS TITLE
// ----------------

if (  (styling2[20]!= null) &&  (styling2[20].toLowerCase() == "hide")   )   
	{  yax.getTitle().setVisible(false);   } 

if (yax.getTitle().isVisible()  )   // 220
{ 

YAxisTitleCaption = yax.getTitle().getCaption() // Get caption of Y AXIS TITLE

if (styling2[21]!= null) { YAxisTitleCaption.setValue(styling2[21]);  } // Title Text
if (styling2[22]!= null) { YAxisTitleCaption.getFont().setName(styling2[22]) } // Font Face
if (styling2[23]!= null) { YAxisTitleCaption.getFont().setSize(parseInt(styling2[23]) ) } // Font Size
if (styling2[24]!= null) { YAxisTitleCaption.setColor( cdvRGBtoColorDefinitionImpl(styling2[24])  ) } // Font Color
if (styling2[25]!= null) { yax.getTitle().setBackground( cdvRGBtoColorDefinitionImpl(styling2[25])  ) } // BG Color

if (styling2[26]!= null) 
{
	fontstyle = styling2[26].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )	{ YAxisTitleCaption.getFont().setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )	{ YAxisTitleCaption.getFont().setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 )	{ YAxisTitleCaption.getFont().setUnderline(true); }
}

if (styling2[27]!= null) { XAxisLabelsCaption.getFont().setRotation(parseInt(styling2[27]) ) } // Rotation angle 567

}  // END IF 220

// A X I S    L A B E L S 
// ======================
// eg: 100, 1000, 10000 for Y-Axis
// or Boston  New York, Washington for X-Axis

// SET X-AXIS LABELS
// -----------------
if (  (styling2[30]!= null) &&  (styling2[30].toLowerCase() == "hide")   )   {  xax.getLabel().setVisible(false);  } 

if (xax.getLabel().isVisible()) // IF 230
{ 
XAxisLabelsCaption = xax.getLabel().getCaption(); // Get caption
if (styling2[32]!= null) { XAxisLabelsCaption.getFont().setName(styling2[32]) } // Font Face
if (styling2[33]!= null) { XAxisLabelsCaption.getFont().setSize(parseInt(styling2[33]) ) } // Font Size
if (styling2[34]!= null) { XAxisLabelsCaption.setColor( cdvRGBtoColorDefinitionImpl(styling2[34])  ) } // Font Color
if (styling2[35]!= null) { xax.getLabel().setBackground( cdvRGBtoColorDefinitionImpl(styling2[35])  ) } // BG Color

if (styling2[36]!= null) 
{
	XAxisLabelsCaption = xax.getLabel().getCaption().getFont(); // Get caption
	fontstyle = styling2[36].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )		{ XAxisLabelsCaption.setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )		{ XAxisLabelsCaption.setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 )	{ XAxisLabelsCaption.setUnderline(true); }
}

// ROTATION ANGLE - X AXIS LABELS
// ==============================
if (styling2[37]!= null) { XAxisLabelsCaption.getFont().setRotation(parseInt(styling2[37]) ) } // Rotation angle 567

// STAGGERED LABELS - X AXIS
// =========================
if  ( (styling2[38]!= null) && ( styling2[38].toLowerCase() == "y"))  { xax.setStaggered(true) }

// =============================
// MULTI LINE X AXIS LABELS
// =============================
if (styling2[39]!= null) 	{ pML = styling2[39]; }  // ML indicator

if(pML != null) // Are multi-line X axis labels required?
{
// Multi-line X axis labels are required.
// =======================================
	 
// Get the text of each X Axis label. If the user has requested
// multi-line labels, these will be used to insert new lines and space characters.  
//

var xax_RunTimeSeries = xaxSeries.getRunTimeSeries(); 	
var xax_rseries0 	= xax_RunTimeSeries.get(0); 		
var xax_Labels = xax_rseries0.getDataSet().getValues();	
var xax_LabelsCount = xax_Labels.length;
var pML_LeftAdjust = false;

if(  (chartOrientation == "Horizontal") && (pML == "left")  )
{
	pML_LeftAdjust = true;
} 

//
// MULTI-LINE X AXIS LABELS are required
// ======================================

// Choices are:
// 1) Multi-line and centered text
// 2) Multi-line and left justified text
// 
// Loop thru the X axis labels and insert new lines where spaces occur.
// If centered is also required, pad the text with spaces based on character count
// of longest word in the label text.

for ( var i=0; i < xax_LabelsCount ; ++i )      // Loop thru X axis labels FOR-XL27
{
  	var labels_Parts = new Array();	// Array to hold parts of the label text
  	labels_Parts = xax_Labels[i].split(' ');	// Split the label on spaces chars. in the text.

    var maxleng;	// Holds number chars in the longest word in the label text

	// LOOP thru label parts array and discover longest word.

	for ( var k=0, len = labels_Parts.length;   k < len;   k++ )
	{
  		if (k == 0)  maxleng = labels_Parts[k].length()
  		else if(labels_Parts[k].length() > maxleng) maxleng = labels_Parts[k].length();
   	}
 
// joinedup holds the combined label parts with newline chars inserted
// (and optionally padding if cetered text is requested)
var joinedup = '';
var padded;	

for ( var d=0, len=labels_Parts.length; d<len; d++ )
{
   var labpart = labels_Parts[d];
   padded = labpart;
   
 // CENTERED LABELS
 // ===============
   
  if(pML == "centered") // Centered - insert padding into text
   {
   if (chartOrientation == "Vertical")
   		{ 	
   		padded =  padLabel( icsc, labpart,  maxleng );
   		}
	if (chartOrientation == "Horizontal")
		{
		padded =  padLabel( icsc, labpart,  maxleng ) ;
		
		// Insert additional padding on right side.
		var indent =  xax_LabelsCount - (xax_LabelsCount - i) + 1 ;
	    backPadding = repeatChar("\u00A0", indent );
 	    padded = padded + backPadding;
		}
   } 

// LEFT and HORIZONTAL ADJUSTMENT
// ==============================
// Adjust if left aligned *and* horizontal orientation
// Insert spaces at the end of each part to force labels to align to the left

if (pML_LeftAdjust)  // IF LFIX
{ 
	var indent =  xax_LabelsCount - (xax_LabelsCount - i) + 1 ;
	backPadding = repeatChar("\u00A0", indent );
 	padded = labpart + backPadding;

} //  END-IF LFIX

// Append a newline char at the start of each label part, 
// * except for 1st part, which is the 1st line * 
   if (d > 0) 	{ padded =  "\u000A" + padded  }	

   labels_Parts[d] = padded; // Save altered label part
}

// Join up the parts of the label into in single string
joinedup = labels_Parts.join('');
// Finally, put the altered label back into the series for X-AXIS labels
xax_Labels[i] = joinedup;

} // END FOR-XL27 : End Loop thru X labels

} // END ML LABELS

} // END IF 230


// SET Y-AXIS LABELS
// -----------------
if (  (styling2[40]!= null) &&  (styling2[40].toLowerCase() == "hide") )  { yax.getLabel().setVisible(false); } 

if (yax.getLabel().isVisible()) // IF 240 - Y AXIS LABEL
{
YAxisLabelsCaption = yax.getLabel().getCaption(); // Get caption
if (styling2[42]!= null) { YAxisLabelsCaption.getFont().setName(styling2[42]) } // Font Face
if (styling2[43]!= null) { YAxisLabelsCaption.getFont().setSize(parseInt(styling2[43]) ) } // Font Size
if (styling2[44]!= null) { YAxisLabelsCaption.setColor( cdvRGBtoColorDefinitionImpl(styling2[44])  ) } // Font Color
if (styling2[45]!= null) { yax.getLabel().setBackground( cdvRGBtoColorDefinitionImpl(styling2[45])  ) } // BG Color

if (styling2[46]!= null)
{
	YAxisLabelsCaption = yax.getLabel().getCaption().getFont(); // Get caption
	fontstyle = styling2[46].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )		{ YAxisLabelsCaption.setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )		{ YAxisLabelsCaption.setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 )	{ YAxisLabelsCaption.setUnderline(true); }
}

// FORMAT SPEC. for Y axis labels
// ==============================
if (styling2[49]!= null) { applyFormatSpecifier(styling2[49], yax, icsc)  }

} // END-IF 240 - Y AXIS LABEL

// SERIES LABELS - TOP of EACH BAR
// ===============================
// Labels which appear on top on each bar 

if (styling2[100]!= null) {   // SERIES LABELS - TOP of EACH BAR
if (styling2[100].toLowerCase() == "show")
{

// SAVE SETTINGS for SERIES LABELS
// ===============================

	pSeriesLabelsShow = true;		// Show series labels (top of each bar etc...)
if (styling2[102]!= null) { pSeriesLabelsFontFace	= styling2[102]				}  // Font Face
if (styling2[103]!= null) { pSeriesLabelsFontSize	= parseInt(styling2[103])	}  // Font Size
if (styling2[104]!= null) { pSeriesLabelsFontColor	= cdvRGBtoColorDefinitionImpl(styling2[104]) } // Font Color
if (styling2[105]!= null) { pSeriesLabelsBGColor	= cdvRGBtoColorDefinitionImpl(styling2[105]) } // BG Color
if (styling2[107]!= null) { pSeriesLabelsRotation   = parseInt(styling2[107]); } 

if (styling2[106] != null) 
{
	fontstyle = styling2[106].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )		{ pSeriesLabelsBold = true; }
	if( fontstyle.indexOf("italic") > -1 )		{ pSeriesLabelsItalic = true;  }
	if( fontstyle.indexOf("underline") > -1 )	{ pSeriesLabelsUnderline = true; }
}
// Label position
// --------------
pSeriesLabelsPosition = "outside"; 	// Default

if (styling2[108] != null) 
{
	llpos = styling2[108].toLowerCase();
	if( llpos.indexOf("inside") > -1 )		{ pSeriesLabelsPosition = "inside"; }
	if( llpos.indexOf("above") > -1 )		{ pSeriesLabelsPosition = "above"; }
	if( llpos.indexOf("below") > -1 )		{ pSeriesLabelsPosition = "below"; }
	if( llpos.indexOf("left") > -1 )		{ pSeriesLabelsPosition = "left"; }
	if( llpos.indexOf("right") > -1 )		{ pSeriesLabelsPosition = "right"; }
}

}} // END SERIES LABELS

}  // IF-103b End
} // END (styling2 != null)
 
// END OF user provided styling 1 & 2
// ==================================

// SET CHART AREA COLORS
// =====================
// p01ChartAreaColor = Chart Block Area
// p02AreaInclAxesColor = Area Including Axes (aka Plot area)
// p03AreaWithinAxesColor = Area within Axes (aka Client Area)
// chart.getBlock().setBackground( cdvCopyInstance(p01ChartAreaColor) );

chart.getBlock().setBackground( cdvCopyInstance(p01ChartAreaColor) );
chart.getPlot().setBackground( cdvCopyInstance(p02AreaInclAxesColor) );
chart.getPlot().getClientArea().setBackground( cdvCopyInstance(p03AreaWithinAxesColor) );

if ( (styling1 != null) && (styling1[21]  != null)    ) 
{
	// Gradient shade the chart block area
	// ===================================
	
		color1 = cdvCopyInstance(p01ChartAreaColor);
		color2 = cdvCopyInstance(p01ChartAreaColor);
		color2 = color2.brighter();
		color2 = color2.brighter();

		// Get direction of gradient
		dir = styling1[21].toLowerCase();
		if (dir == "lr") gradDegrees = 0;
		if (dir == "rl") gradDegrees = 0;
		if (dir == "bt") gradDegrees = 90;
		if (dir == "tb") gradDegrees = -90;

		var newGrad2  = GradientImpl.create(color1, color2, gradDegrees, false);
		chart.getBlock().setBackground( newGrad2 ); // Gradient color
} 

// PIE CHART 
// =========

if (chartMode == "WITHOUTAXES") // IF PIE 209
{
	pcSeriesDefsAll			= chart.getSeriesDefinitions();   // 1
	pcSeriesDefs0 			= pcSeriesDefsAll.get(0);  // 2
	
	pcSeriesDefsAllInner 	= pcSeriesDefs0.getSeriesDefinitions();  // 3
	pcSeriesDefsInner0		= pcSeriesDefsAllInner.get(0);  // 4
	mainSeries1 			= pcSeriesDefsInner0.getRunTimeSeries().get(0); // 5
	
	// SET DATA POINT LABELS characteristics - DEFAULT
	mainSeries1.getLabel().getCaption().getFont().setName("San Serif");
	mainSeries1.getLabel().getCaption().getFont().setSize(8);
	mainSeries1.getLabel().getCaption().setColor(ColorDefinitionImpl.create(60,60,60));

// APPLY PIE CHART CUSTOM 
// ======================

if(styling1 != null)		// Check that styling1 is defined  IF 124
{
		// Set Explode Distance
		if (styling1[141] != null) mainSeries1.setExplosion(parseInt(styling1[141]) );
		// Set Explode Formula
		if (styling1[142] != null) mainSeries1.setExplosionExpression( styling1[142] );
		// Ratio (Tilt angle)
		if (styling1[143] != null) mainSeries1.setRatio(  parseFloat(styling1[143]) );
		// Rotation angle
		if (styling1[144] != null) mainSeries1.setRotation( parseFloat(styling1[144]) ); 
		// Segment outline color
		if (styling1[145]!= null) mainSeries1.setSliceOutline( cdvRGBtoColorDefinitionImpl(styling1[145])  );
		
		// Lines to Data point labels (known as: Lead Lines)
		// Set Line thickness
		if (styling1[146] != null) 
			mainSeries1.getLeaderLineAttributes().setThickness(parseInt(styling1[146]) );
			// Set Line Color
		if (styling1[147] != null) 
			mainSeries1.getLeaderLineAttributes().setColor(  cdvRGBtoColorDefinitionImpl(styling1[147])   );
			// Set Line length (10 = default.  0-9 shorter.  11+ longer)
		if (styling1[148] != null) 
			mainSeries1.setLeaderLineLength(parseInt(styling1[148]) );
		
} // END IF 124
	
// PIE CHART - DATA POINT LABELS
// =============================

if (styling2 != null) {	// IF 207

if (styling2[100]!= null) {
if (styling2[100].toLowerCase() == "show")
{
llcaption = mainSeries1.getLabel().getCaption();

if (styling2[102]!= null) { llcaption.getFont().setName(styling2[102]) }  // Font Face
if (styling2[103]!= null) { llcaption.getFont().setSize(parseInt(styling2[103]) ) }  // Font Size
if (styling2[104]!= null) { llcaption.setColor( cdvRGBtoColorDefinitionImpl(styling2[104])  ) } // Font Color
if (styling2[105]!= null) { mainSeries1.getLabel().setBackground( cdvRGBtoColorDefinitionImpl(styling2[105])  ) } // BG Color
if (styling2[106] != null) 
{

	llcaption = llcaption.getFont();
	fontstyle = styling2[106].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )		{ llcaption.setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )		{ llcaption.setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 )	{ llcaption.setUnderline(true); }
}
// [107] Rotation angle
if (styling2[107]!= null) { mainSeries1.getLabel().getCaption().getFont().setRotation(parseInt(styling2[107])) } 
if (styling2[108] != null) 
		{
			if (styling2[108].toLowerCase() == "inside") mainSeries1.setLabelPosition( Position.INSIDE_LITERAL );
			else
		 	mainSeries1.setLabelPosition( Position.OUTSIDE_LITERAL );
		}

// CHECK PIE CHART is PERCENTAGE SEGMENTS
// ======================================
if (styling2[109]!= null)	// IF FORMSPEC109 
{  	
pieTypeFormatSpec = styling2[109] + ","	// Put comma on end so split will work
pieFormParts = pieTypeFormatSpec.split(',');

// IF SUFFIX IS "%", 
// ALTER PIE CHART TO PERENTAGE PIE CHART

if (pieFormParts[1] == "%")
{
	dpoint = mainSeries1.getDataPoint().getComponents();  // Get Data Point components 
	dpx = dpoint.get(0);
	dpx.setType(DataPointComponentType.PERCENTILE_ORTHOGONAL_VALUE_LITERAL);	// Set type of datapoint
	applyFormatSpecifier(",%,100,2", dpx, icsc)  

} 
else
{  	
	dpoint = mainSeries1.getDataPoint().getComponents();	// Get Data Point components 
	dpx = dpoint.get(0);
	applyFormatSpecifier(styling2[109], dpx, icsc)  
}

}	// END IF FORMSPEC109
	   	
}}}
	
} // END IF PIE 209

// =========================================
// BAR/LINE/AREA charts (CHARTS with AXES) 
// =========================================

if (chartMode == "WITHAXES")  // IF-202
{
// CLIENT AREA (within PLOT) OUTLINE COLOR
// =======================================
chart.getPlot().getClientArea().getOutline().setVisible(true);
chart.getPlot().getClientArea().getOutline().setColor( cdvCopyInstance(pLineColorD) );
chart.getPlot().getClientArea().getOutline().setThickness(0); 
	

// Y AXIS ticks (for better aesthetics)
// ====================================
// Set the tick color to the same color as the plot area 
// so they are effectively invisible, but the spacing is kept between the 
// labels and the Y-AXIS.
// Else, we would need to insert a space to the right of the Y AXIS labels

// Only apply default tick settings if user did *not* supply any X or Y tick
// styling choices. 

doDefaultTicks = false;		// Flag to indictae of to do tick default code

if  (styling1 == null)  doDefaultTicks = true;
if ((styling1    != null)	&&
	(styling1[72] == null)	&&  
	(styling1[82] == null)  )
		{ doDefaultTicks = true;}

if ( doDefaultTicks == true )   // IF 277
{  
// If here....Apply Tick default settings. 

pTickMarkShowY = "show";
pTickMarkColorY = cdvCopyInstance(p02AreaInclAxesColor);

// Set Tick mark settings based in chart orientation
// -------------------------------------------------
if(chartOrientation == "Horizontal")
{
pTickMarkColorY = cdvCopyInstance(pTickMarkColorDefault);
pTickMarkColorX = cdvCopyInstance(p02AreaInclAxesColor);
}
else
{
pTickMarkColorY = cdvCopyInstance(p02AreaInclAxesColor);
pTickMarkColorX = cdvCopyInstance(pTickMarkColorDefault);
}

} // END IF 277

// LOOP THRU ALL Y AXES
// ====================
// The nested for loops, process the following:
//
// for (each Y Axis associated with the X Axis)
//     for (this Y Axis, loop thru each series defintions attached to the Y-Axis)
//        for (loop thru each series in the series defintion)
//              Action: for each series, set certain visual elements
//              eg: Line Series, set the marker to square, size 2
//              Area Series, set the separator line between the areas
//              Bar series, remove the outline on the riser (aka bar)
//
 
// LOOP THRU each Y Axis that is *associated* with the X Axis.
// ===========================================================
// Usually there is only 1 Y Axis, on the left side of the X Axis.
// However, you can have multiple Y Axis, usuallly 2: On left and right side of chart.
// Each Y Axis can represent different values.
// 
var asxloopct;      // Assoc. Axes loop counter
var asxLoopIndex;   // Assoc. Axes loop index.

// assocAxes.size() = Number of y Axes

for (asxloopct = 0 ;asxloopct < assocAxes.size() ; asxloopct++)	

{  // Next Y Axis :  YX708 for loop

	yax = assocAxes.get(asxloopct);  // Get the next Y axis series defns
    
 
// GET SERIES DEFINITIONS -  for current Y AXIS 
// ============================================
	
// =============================================================
// Loop thru each SERIES DEFINITION
// and access each SERIES within each SERIES DEFN.
// Each SERIES contains:
// 1) the RISER OUTLINE color (Bar Charts only) 
// 2) the SERIES IDENTIFIER
// ==============================================================

var y2loopct;
var y2SeriesLoopIndex;

for (y2loopct = 0 ; y2loopct < yax.getSeriesDefinitions().size(); y2loopct++)	// FOR-101
{

    // LOOP thru Y AXIS Series Defintions  START- for loop SD101
    // ==============================================================
	// GET number of SERIES in the SERIES DEFN.
  	// The +0 series in the array is not used.
  	// So start from the +1 series. See y2SeriesLoopIndex = 1.
  	
	y2UniqueSeriesDef      =  yax.getSeriesDefinitions().get(y2loopct);
	y2SeriesSize           =  y2UniqueSeriesDef.getSeries().size();
	
	// LOOP thru each ENTRY in this SERIES 
	// ===================================
	
	for (y2SeriesLoopIndex = 1; y2SeriesLoopIndex < y2SeriesSize; y2SeriesLoopIndex++  ) // START for loop SER-Y102
	{
        
		y2Series = y2UniqueSeriesDef.getSeries().get(y2SeriesLoopIndex);
		
    	
    	// HERE...we have a SERIES
    	//
    	// SET THE DATA POINT SERIES LABELS 
    	// ================================
    	// (based on custom styling...if any)
    	
    	if (pSeriesLabelsShow == true)	// IF 727
    	{
    		
   			y2Series.getLabel().setVisible(true);
			llcaption = y2Series.getLabel().getCaption();  // Get label caption
			llcaption.getFont().setName(pSeriesLabelsFontFace);		// Font Face
			llcaption.getFont().setSize(pSeriesLabelsFontSize );	// Font Size
			llcaption.setColor( cdvCopyInstance(pSeriesLabelsFontColor) ); // Font Color
			llcaption.getFont().setRotation(pSeriesLabelsRotation);	// Font Rotation angle
			
			// Copy BG color for each label(Must copy. Simply refering to color only works for 
			// the first label.)
			y2Series.getLabel().setBackground( cdvCopyInstance(pSeriesLabelsBGColor) ); // BG Color
			
			llposition = y2Series.getLabelPosition();
			
    		// Bold,Italic,Underline
    		if (pSeriesLabelsBold) llcaption.getFont().setBold(true);
    		if (pSeriesLabelsItalic) llcaption.getFont().setItalic(true);
    		if (pSeriesLabelsUnderline) llcaption.getFont().setUnderline(true);
    			
    		///Data Point Label - Position
			switch (pSeriesLabelsPosition)
			{
				case "inside":	y2Series.setLabelPosition(Position.INSIDE_LITERAL); break;
				case "outside": y2Series.setLabelPosition(Position.OUTSIDE_LITERAL); break;
				case "above": 	y2Series.setLabelPosition(Position.ABOVE_LITERAL); break;
				case "below": 	y2Series.setLabelPosition(Position.BELOW_LITERAL); break;
				case "left": 	y2Series.setLabelPosition(Position.LEFT_LITERAL); break;
				case "right": 	y2Series.setLabelPosition(Position.RIGHT_LITERAL); break;
    			default:	y2Series.setLabelPosition(Position.INSIDE_LITERAL); break;
			} // End switch


    	   	// Series Labels format specifier  
    	   	// ==============================
    	   	if (styling2[109]!= null)
    	   	{  	
    	   	    dpoint = y2Series.getDataPoint().getComponents();	// Get Data Point components 
    	   	    dpx = dpoint.get(0);
    	   	    applyFormatSpecifier(styling2[109], dpx, icsc)  
    	   	}
	   
	       // Series Datapoint Labels : If Percentage stacked bar chart
    	   // ==========================================================    
	    
			if (chartSubType == "Percent Stacked")
			{
	 			dpoint = y2Series.getDataPoint().getComponents();	// Get Data Point components 
     			dpx = dpoint.get(0);
     			dpx.setType(DataPointComponentType.PERCENTILE_ORTHOGONAL_VALUE_LITERAL);	// Set type of datapoint
     			applyFormatSpecifier(",%,100,2", dpx, icsc)   
			} 
    	   	    	
    	   // Series Datapoint Labels : Insets - Fine tuning
    	   // ===============================================    
			if(styling2[101] != null)  // IF-IN101
			{
				insetsSL = y2Series.getLabel().getInsets();
				applyInsets(styling2[101], insetsSL , icsc);  
			}  // END IF-101	
    	    	   	      	
    	}	// END-IF 727 show series labels
    	
    	// BAR CHARTS - Bar outline color
    	if (chartType == "Bar Chart")	// SET RISER OUTLINE Color
    		{  y2Series.setRiserOutline(cdvCopyInstance(pBarOutLine));  } // Set BAR Outline
    
    
    
    	// AREA CHARTS - Area Separator color
		if (chartType == "Area Chart")  // IF-AREA_222
		{
				var llArea = y2Series.getLineAttributes();	// Get series Line Attributes
				llArea.setVisible(true); 
			    llArea.setColor(cdvCopyInstance(pAreaSeparator) );
			   
		} // End IF-AREA_222
		
// LINE CHARTS
// ===========

if (chartType == "Line Chart")  // IF-LINE-333
{
// LINE MARKERS  
// ============	
var llMarker = y2Series.getMarkers().get(0);

// LINE THICKNESS 
// The actual line AND the line that surrounds the marker 
// ========================================================
la = y2Series.getLineAttributes();
la.setThickness(pLineThickness);
la.setColor(  cdvCopyInstance(pScheme[y2SeriesLoopIndex -1 ])  );

if ( pLineMarkerSize == 0 )
{
	// MARKERS HIDDEN
	// No markers are displayed, so shade the legend line
	// to match the line charts
}

// SET MARKERS
// ===========
var llMarker = y2Series.getMarkers().get(0);

if (llMarker != null)
{	// LLM-101
switch (pLineMarkerShape)
{
	case "box": llMarker.setType(MarkerType.BOX_LITERAL); break;
	case "circle": llMarker.setType(MarkerType.CIRCLE_LITERAL); break;
	case "crosshair": llMarker.setType(MarkerType.CROSSHAIR_LITERAL); break;
	case "diamond": llMarker.setType(MarkerType.DIAMOND_LITERAL); break;
	case "star": llMarker.setType(MarkerType.STAR_LITERAL); break;
	case "triangle": llMarker.setType(MarkerType.TRIANGLE_LITERAL); break;
    default: llMarker.setType(MarkerType.BOX_LITERAL); break;
} // End switch

// MARKER SIZE
// ============
 	llMarker.setSize(pLineMarkerSize);   // Set marker size

} // END LLM-101   909	
} // END IF-LINE-333 - Line Charts
	
 
	 }	// END for loop SER-Y102  
   }  //   END - for loop SD101
} // End YX708 for looop of each associated Y Axis


// CHARTS WITH AXES + 2D_DEPTH : Extra processing
// ==============================================

if (chartDim == "2D_DEPTH" )
{
	
// Here if chart is 2D with Depth
// ------------------------------
// Shade the client area the same color as lighter stripe color
// This is needed to fill the gap between the chart floor and 
// the bottom of the first marker range.
// 1) Set Floor color
// 2) Set Side Wall color

	chart.getPlot().getClientArea().setBackground(cdvCopyInstance(pLighterColor) ) ;
	chart.setFloorFill( cdvCopyInstance(pFloorColor) );		// 1
	chart.setWallFill ( cdvCopyInstance(pSideWallColor) ); 	// 2

} // END - 2D with DEPTH special processing

//================================================= 
// ALL CHARTS with AXES HERE
// Both: 2D_PLAIN & 2D_DEPTH
// ================================================

// TICK MARKS : Set X axis TICKS to Below, Set Y axis TICKS to Left
// =================================================================

// X AXIS TICK MARKS
// -----------------

if (pTickMarkShowX == "show")
{	// IF 598
xax.getMajorGrid().getTickAttributes().setVisible(true);

if	(pTickMarkPositionX == "below") xax.getMajorGrid().setTickStyle(TickStyle.BELOW_LITERAL);
else if	(pTickMarkPositionX == "across") xax.getMajorGrid().setTickStyle(TickStyle.ACROSS_LITERAL);
else if	(pTickMarkPositionX == "above") xax.getMajorGrid().setTickStyle(TickStyle.ABOVE_LITERAL);
xax.getMajorGrid().getTickAttributes().setColor(pTickMarkColorX);
xax.getMajorGrid().getTickAttributes().setThickness(pTickMarkWidthX);
// Test SOLID first. It's the default and most common
if      ( pTickMarkStyleX == "solid" ) xax.getMajorGrid().getTickAttributes().setStyle( LineStyle.SOLID_LITERAL );
else if ( pTickMarkStyleX == "dotted" ) xax.getMajorGrid().getTickAttributes().setStyle( LineStyle.DOTTED_LITERAL );
else if ( pTickMarkStyleX == "dashed" ) xax.getMajorGrid().getTickAttributes().setStyle( LineStyle.DASHED_LITERAL );
else if ( pTickMarkStyleX == "dashdotted" ) xax.getMajorGrid().getTickAttributes().setStyle(LineStyle.DASH_DOTTED_LITERAL );
}
else
{
xax.getMajorGrid().getTickAttributes().setVisible(false);
} // END-IF 598

// Y AYIS TICK MARKS ptttt
// ----------------- 

if (pTickMarkShowY == "show")  // IF 599
{
yax.getMajorGrid().getTickAttributes().setVisible(true);
yax.getMajorGrid().setTickStyle(TickStyle.LEFT_LITERAL);

if	(pTickMarkPositionY == "left") yax.getMajorGrid().setTickStyle(TickStyle.LEFT_LITERAL);
else if	(pTickMarkPositionY == "across") yax.getMajorGrid().setTickStyle(TickStyle.ACROSS_LITERAL);
else if	(pTickMarkPositionY == "right") yax.getMajorGrid().setTickStyle(TickStyle.RIGHT_LITERAL);
yax.getMajorGrid().getTickAttributes().setColor(pTickMarkColorY);
yax.getMajorGrid().getTickAttributes().setThickness(pTickMarkWidthY);

// Test SOLID first. It's the default and most common
if      ( pTickMarkStyleY == "solid" ) yax.getMajorGrid().getTickAttributes().setStyle( LineStyle.SOLID_LITERAL );
else if ( pTickMarkStyleY == "dotted" ) yax.getMajorGrid().getTickAttributes().setStyle( LineStyle.DOTTED_LITERAL );
else if ( pTickMarkStyleY == "dashed" ) yax.getMajorGrid().getTickAttributes().setStyle( LineStyle.DASHED_LITERAL );
else if ( pTickMarkStyleY == "dashdotted" ) yax.getMajorGrid().getTickAttributes().setStyle(LineStyle.DASH_DOTTED_LITERAL );
} else
{
	yax.getMajorGrid().getTickAttributes().setVisible(false);
}
// END IF 599


// If 2nd Y AXIS, set Tick Marks also
if (yax2 != null)
{

yax2.getMajorGrid().getTickAttributes().setVisible(true);
yax2.getMajorGrid().setTickStyle(TickStyle.RIGHT_LITERAL);
//
yax2.getMajorGrid().getTickAttributes().setColor( ColorDefinitionImpl.GREEN() );
yax2.getMajorGrid().getTickAttributes().setThickness(0);

// Test SOLID first. It's the default and most common
if      ( pTickMarkStyleY == "solid" ) yax2.getMajorGrid().getTickAttributes().setStyle( LineStyle.SOLID_LITERAL );
else if ( pTickMarkStyleY == "dotted" ) yax2.getMajorGrid().getTickAttributes().setStyle( LineStyle.DOTTED_LITERAL );
else if ( pTickMarkStyleY == "dashed" ) yax2.getMajorGrid().getTickAttributes().setStyle( LineStyle.DASHED_LITERAL );
else if ( pTickMarkStyleY == "dashdotted" ) yax2.getMajorGrid().getTickAttributes().setStyle(LineStyle.DASH_DOTTED_LITERAL );
}


// PLOT AREA SPACING - Set to none. 
// =================================
// By default there is a gap between the axis lines and 
// the plot area. By setting the spacing to zero, the 
// marker range Stripes reach the edge of the plot area.
// Without this adjustments, the stripes do NOT reach to the Y-AXIS and looks
// unattractive

chart.getPlot().setHorizontalSpacing(0);
chart.getPlot().setVerticalSpacing(0);

// SCALE - Get Y-Axis range and where the ticks are.
// =================================================
var yscale = yax.getScale();

// CALCULATE THE SCALE 
// ===================

if (chartSubType == "Percent Stacked")
{
	yscale.setMin(  NumberDataElementImpl.create(0.0)   ); 	// 0 to 100%
	yscale.setMax(  NumberDataElementImpl.create(100.0) );
} 
else
{
	// Calculate scale based on data series.
	cdvCalculateScale2(chart, yscale, icsc, styling1);
}		

// BACK WALL STRIPES required?  
// ===========================
// Paint alternate shading Stripes (Marker Ranges) on the back wall of the chart

if( pPaintStripes == "Y") 
	{ cdvPaintStripes(  yax, yscale, pLighterColor , pDarkerColor, chartDim, icsc) }  // Yes, stripe the back wall.
	else
	// NO STRIPES
	// ----------
	// Set Back Wall color and side wall color to p03AreaWithinAxesColor.
	{ 
		chart.getPlot().getClientArea().setBackground(p03AreaWithinAxesColor);   // Back wall color
	    chart.setWallFill ( pSideWallColor ); 	// Side wall
	    
		// chart.setFloorFill( cdvCopyInstance(pFloorColor) );
		// chart.setWallFill ( cdvCopyInstance(pSideWallColor) ); 	// 2
	
	}

// BOTH STRIPES and NO STRIPE BAR charts 
// =====================================

// A X I S   L I N E S (Colors)
// ============================

	// X AXIS LINE
	// ===========
	xax.getLineAttributes().setVisible(true);
	xax.getLineAttributes().setColor( cdvCopyInstance(pLineColorD)   );
	xax.getLineAttributes().setThickness(0.5);
	
	// Y AXIS LINE 
	// ===========
	yax.getLineAttributes().setVisible(true);
	yax.getLineAttributes().setColor(cdvCopyInstance(pLineColorD)  );
	yax.getLineAttributes().setThickness(0.5);
	
	// 2nd Y AXIS
	// ==========
	if (yax2 != null )
	{
	yax2.getLineAttributes().setVisible(true);
	yax2.getLineAttributes().setColor(cdvCopyInstance(pLineColorD)  );
	yax2.getLineAttributes().setThickness(0.5);
	
	}

// ===========================
// AXIS LINE - CUSTOM STYLIING
// ===========================

if (styling1 != null)  // IF CSAXIS - Ensure stlying1 is present 
{
// X AXIS LINE
xaxLine = xax.getLineAttributes();
if (styling1[55] != null) { xaxLine.setThickness( parseInt(styling1[55]) ); }  // Thickness
if (styling1[56] != null) { xaxLine.setColor( cdvRGBtoColorDefinitionImpl(styling1[56]) );  } // Line Color 

// Y AXIS LINE
yaxLine = yax.getLineAttributes();
if (styling1[65] != null) { yaxLine.setThickness( parseInt(styling1[65]) ); }  // Thickness
if (styling1[66] != null) { yaxLine.setColor( cdvRGBtoColorDefinitionImpl(styling1[66]) );  } // Line Color

} // END IF CSAXIS


// GRID LINES - MAJOR X AXIS (Usually Vertical line)
// ===================================================
	
	if(chartOrientation == "Horizontal")
	{
	 	pGridLineShowX == "hide";
	 	pGridLineShowY == "show";
	 	
	 	majorGridAttributes = xax.getMajorGrid().getLineAttributes();  // X AXIS MAJOR GRID
	    majorGridAttributes.setVisible(false);  // Hide X
	    
	    majorGridAttributes = yax.getMajorGrid().getLineAttributes(); // Y AXIS MAJOR GRID
		majorGridAttributes.setVisible(true);  // Show Y
     }		

    if(pGridLineShowX == "show")
    {
	var majorGridAttributes = xax.getMajorGrid().getLineAttributes();  // X AXIS MAJOR GRID
	majorGridAttributes.setVisible(true); 
	majorGridAttributes.setColor( cdvCopyInstance(pGridLineColorX) );
	majorGridAttributes.setThickness( parseFloat( pGridLineWidthX) ); // Set width of gridline
	
	// Test SOLID first. It's the default and most common
	if      ( pGridLineStyleX == "solid" ) majorGridAttributes.setStyle( LineStyle.SOLID_LITERAL );
	else if ( pGridLineStyleX == "dotted" ) majorGridAttributes.setStyle( LineStyle.DOTTED_LITERAL );
	else if ( pGridLineStyleX == "dashed" ) majorGridAttributes.setStyle( LineStyle.DASHED_LITERAL );
	else if ( pGridLineStyleX == "dashdotted" ) majorGridAttributes.setStyle( LineStyle.DASH_DOTTED_LITERAL );
	}
	
	// GRID LINES - MAJOR Y AXIS (Usually Horizontal line) 
    // ===================================================
    
    if(pGridLineShowY != "hide")
    {
    var majorGridAttributes = yax.getMajorGrid().getLineAttributes(); // Y AXIS MAJOR GRID
    
	majorGridAttributes.setVisible(true); 
	majorGridAttributes.setColor( cdvCopyInstance(pGridLineColorY) );
	majorGridAttributes.setThickness( parseFloat( pGridLineWidthY) ); // Set width of gridline
	
	// Test SOLID first. It's the default and most common
	if      ( pGridLineStyleY == "solid" ) majorGridAttributes.setStyle( LineStyle.SOLID_LITERAL );
	else if ( pGridLineStyleY == "dotted" ) majorGridAttributes.setStyle( LineStyle.DOTTED_LITERAL );
	else if ( pGridLineStyleY == "dashed" ) majorGridAttributes.setStyle( LineStyle.DASHED_LITERAL );
	else if ( pGridLineStyleY == "dashdotted" ) majorGridAttributes.setStyle( LineStyle.DASH_DOTTED_LITERAL );
	}

// MASK THE TOP GRID LINE - If the grid line is not solid eg: dotted or dashed 
// ============================================================================
// The top grid line sits across the top of the chart plot area.
// A non-solid line across the top of the plot area looks unattractive.
// So, a marker line can be created and drawn over the top grid line to mask/hide the dotted grid line.
// This is only possible if the chart (WITH AXES) is 2D plain (NOT with depth).
// Also, only apply the marker line if the grid lines are NOT solid, and the user has requested this feature.
// Default behavior is to not draw the maker line.
// This is especially valuable when darker back wall colors are used. 

if(	(chartDim == "2D_PLAIN")	// IF TOP GRID LINE
&&  (chartMode == "WITHAXES")
&&	(pGridLineStyleY != "solid")  
&&	(pGridLineHideTop == "y" )  )  
{ 
	topLineValue = NumberDataElementImpl.create( yscale.getMax().getValue() );  // Get value of top grid line
	// Create marker line (same color as grid line)
	mlyax = MarkerLineImpl.create( yax, topLineValue , cdvCopyInstance(pGridLineColorY) ); 
	// mlyax = MarkerLineImpl.create( yax, topLineValue , ColorDefinitionImpl.BLACK() ); // TTT
	mlyax.getLineAttributes().setStyle( LineStyle.SOLID_LITERAL ); // Set solid style
	mlyax.getLineAttributes().setThickness( pGridLineWidthY );  // Same as grid line thickness
	mlyax.getLabel().setVisible(false);  // Hide line label
}  // END - IF TOP GRID LINE 

} // End (chartMode == "WITHAXES") END IF-101


// =================
// L E G E N D
// =================
hLegend = chart.getLegend(); // GET HANDLE to LEGEND

cdvSetLegendDefaults(hLegend); // Set Legend Defaults

if(styling2 != null)  // IF 601 - Check if user has provided s2 options
{
pShowLegend = true;
if (   (styling2[170]!= null) && (styling2[170].toLowerCase() ) == "hide" ) 
	{ 
		hLegend.setVisible(false); // Hide Legend
		pShowLegend = false;
	}

// LEGEND STYLING
// ============== 
if (pShowLegend == true)
{			// IF pShowLegend is true

// L E G E N D   T I T L E 
// =======================
if (styling2[80] == "show") 	// IF LEGTITLE-87
{
hLegend.getTitle().setVisible(true);	// Ensure legend title is visible
	
if (styling2[81] != null) { hLegend.getTitle().getCaption().setValue(styling2[81]); }  // Text
if (styling2[82] != null) { hLegend.getTitle().getCaption().getFont().setName(styling2[82]); }  // Font Face
if (styling2[83] != null) { hLegend.getTitle().getCaption().getFont().setSize(parseInt(styling2[83]));  }  // Font Size
if (styling2[84] != null) { hLegend.getTitle().getCaption().setColor(cdvRGBtoColorDefinitionImpl(styling2[84]) )    }
if (styling2[85] != null) { hLegend.setBackground( cdvRGBtoColorDefinitionImpl(styling2[85]) )  } // BG Color


//  FONT STYLES for ENTRIES
if (styling2[86]!= null) 
{
	fontstyle = styling2[86].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )	  { hLegend.getTitle().getCaption().getFont().setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )	  { hLegend.getTitle().getCaption().getFont().setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 ) { hLegend.getTitle().getCaption().getFont().setUnderline(true); }
}
} // END-IF LEGTITLE-87

// L E G E N D   E N T R I E S
// ===========================
if (styling2[172] != null) { hLegend.getText().getFont().setName(styling2[172]);                           }  // Font Face
if (styling2[173] != null) { hLegend.getText().getFont().setSize(  parseInt(styling2[173])  );        }  // Font Size
if (styling2[174] != null) { hLegend.getText().setColor(cdvRGBtoColorDefinitionImpl(styling2[174]) )    }
if (styling2[175] != null) { hLegend.setBackground( cdvRGBtoColorDefinitionImpl(styling2[175]) )  } // BG Color

// FONT STYLES for ENTRIES
if (styling2[176]!= null) 
{
	fontstyle = styling2[176].toLowerCase();
	if( fontstyle.indexOf("bold") > -1 )	{ hLegend.getText().getFont().setBold(true); }
	if( fontstyle.indexOf("italic") > -1 )	{ hLegend.getText().getFont().setItalic(true); }
	if( fontstyle.indexOf("underline") > -1 )	{ hLegend.getText().getFont().setUnderline(true); }
}

// LEGEND CLIENT AREA
// ==================
if (styling2[177] != null) { hLegend.getClientArea().setBackground( cdvRGBtoColorDefinitionImpl(styling2[177]) )  } // Client area BG Color
if (styling2[178] != null) // Client area Line color
{ 
	hLegend.getClientArea().getOutline().setColor( cdvRGBtoColorDefinitionImpl(styling2[178]) ); 
	chart.getLegend().getClientArea().getOutline().setVisible(true);
} 
if (styling2[179] != null) { hLegend.getClientArea().getOutline().setThickness( parseInt(styling2[179]) ); } // Client area Line thickness


// CUSTOM LEGEND PLACEMENT
// =======================

if ( styling2[181] != null) 
{
legendPosition = styling2[181].toLowerCase();

switch (legendPosition)
{
    case "above":   hLegend.setPosition(Position.ABOVE_LITERAL); break;
    case "below":  hLegend.setPosition(Position.BELOW_LITERAL); break; 
    case "left":   hLegend.setPosition(Position.LEFT_LITERAL); break; 
    case "right":  hLegend.setPosition(Position.RIGHT_LITERAL); break; 
    case "inside":  hLegend.setPosition(Position.INSIDE_LITERAL); break; 
    case "outside":  hLegend.setPosition(Position.OUTSIDE_LITERAL); break;
    default:    hLegend.setPosition(Position.BELOW_LITERAL); break; 
}
}

// LEGEND - CUSTOM ANCHOR SETTING
// ==============================

if ( styling2[182] != null) 
{
legendANCHOR = styling2[182].toLowerCase();

switch (legendANCHOR)
{
case "e":  hLegend.setAnchor(Anchor.EAST_LITERAL); break; 
case "w":  hLegend.setAnchor(Anchor.WEST_LITERAL); break; 
case "n":  hLegend.setAnchor(Anchor.NORTH_LITERAL); break; 
case "s":  hLegend.setAnchor(Anchor.SOUTH_LITERAL); break; 
case "sw": hLegend.setAnchor(Anchor.SOUTH_WEST_LITERAL); break; 
case "se": hLegend.setAnchor(Anchor.SOUTH_EAST_LITERAL); break; 
case "nw": hLegend.setAnchor(Anchor.NORTH_WEST_LITERAL); break; 
case "ne": hLegend.setAnchor(Anchor.NORTH_EAST_LITERAL); break; 
default:  hLegend.setAnchor(Anchor.EAST_LITERAL); break; 
}
}

//  LEGEND ORIENTATION
//  ===================
//  Orientation.HORIZONTAL_LITERAL 
//  Orientation.VERTICAL_LITERAL 

if ( styling2[183] != null) 
{
legendOR = styling2[183].toLowerCase();
if ( legendOR == "horizontal") hLegend.setOrientation(Orientation.HORIZONTAL_LITERAL);  
if ( legendOR == "vertical") hLegend.setOrientation(Orientation.VERTICAL_LITERAL);  
}

//  LEGEND DIRECTION
//  =================
//  Direction.LEFT_RIGHT_LITERAL 
//   Direction.TOP_BOTTOM_LITERAL 

if ( styling2[184] != null) 
{
legendDIRECTION = styling2[184].toLowerCase();
if ( legendDIRECTION == "lr") hLegend.setDirection(Direction.LEFT_RIGHT_LITERAL );  
if ( legendDIRECTION == "tb") hLegend.setDirection(Direction.TOP_BOTTOM_LITERAL );  
}

}	// IF pShowLegend is true
}   // END-IF 601


// INSETS - FINE TUNING
// ====================
// 31: X Axis labels
// 41: Y Axis labels

if   ( (styling2 != null) &&  (styling2[31] != null) )  // IF-IN31
{	// X AXIS LABELS - INSETS
	insetsx31 = xax.getLabel().getInsets(); applyInsets(styling2[31], insetsx31 , icsc);  
}  // END IF-IN31

if   ( (styling2 != null) &&  (styling2[41] != null) )  // IF-IN41
{	// Y AXIS LABELS - INSETS
	insetsy41 = yax.getLabel().getInsets(); applyInsets(styling2[41], insetsy41 , icsc);  
}  // END IF-IN41

// This saves lots of unnecassarty tests.
// 201: Spacing around chart within the plot area
// 202: Spacing around chart *inside* the chart area
// 203: Spacing around the legend block
// 204: Spacing within the legend client area
// 205: Spacing around the legend title

// Confirm styling2 array is present before testing indivual entries.
if ( styling2 != null )  // IF-IN200
{

if (styling2[201] != null) 
{  
insets201 = chart.getPlot().getInsets();
applyInsets(styling2[201], insets201 , icsc);  
}

if (styling2[202] != null) 
{  
insets202 = chart.getPlot().getClientArea().getInsets();
applyInsets(styling2[202], insets202 , icsc);  
}

if (styling2[203] != null) 
{  
insets203 = chart.getLegend().getInsets(); 
applyInsets(styling2[203], insets203 , icsc);  
}

if (styling2[204] != null) 
{  
insets204 = chart.getLegend().getClientArea().getInsets();
applyInsets(styling2[204], insets204 , icsc);  
}

if (styling2[205] != null) 
{  
insets205 = chart.getLegend().getTitle().getInsets();
applyInsets(styling2[205], insets205 , icsc);  
}

} // END IF-IN200

// ===================================================
// CALCULATE the COLOR BY MODE - By CATEGORY of SERIES
// ===================================================

if (chartMode == "WITHAXES")
{
	if  ( (styling1 != null) && (styling1[1] != null) )
		pColorBy = styling1[1];	// Apply what the user wants. Skip built in rules.
	else
	{
		if (ySeriesSize == 2) 
			{ pColorBy = "C";  	} // Only 1 series. Color each bar a separate color.
		else
			{ pColorBy = "S";   } // Multiple series - color each bar by series color
		//endif
		
	}	//end else
}

// ------------------------------------------------
if (chartMode == "WITHOUTAXES")
{
	if  ( (styling1 != null) && (styling1[1] != null) )
		pColorBy = styling1[1];	// Apply what the user wants. Skip built in rules.
	else 
		pColorBy = "C";	// Only 1 series. Color each bar a separate color
}
// -------------------------------------------------

// SET THE COLOR BY MODE 
// =====================

if (pColorBy == "C")
{  chart.getLegend().setItemType(LegendItemType.CATEGORIES_LITERAL);   } 
else	
{  chart.getLegend().setItemType(LegendItemType.SERIES_LITERAL);  }	


// =====================
// POPULATE the PALETTES
// =====================
//
// GRAIDENT colors
// If pBarGradient is not null, 
// Adjust pScheme colors to gradient colors.
//
// styling1[139] values:
// The first character represents the DARK end of the gradient 
// LR : Left Dark --> Right Light
// RL : Right Dark --> Left Light
// BT : Bottom Dark --> Top Light
// TB : Top Dark --> Bottom Light
//
// The current color in the palette is the dark color.
// A lighter (brighter) color is created.

// FILL THE SERIES PALETTE 
// -----------------------
var seriesPal2 = "";

// SINGLE BAR COLOR needed?
// =======================
// Charts with bars all the same color. Use this color when:
// a) The user provides a color for the bars s1[43]
// b) There is only 1 series in the chart ie: (ySeriesSize == 2) -AND-
// c) The user expressly requests BySeries colors: s1[1] = "S" -AND-
// Simply copy the sinlge bar color to the 0th entry of the color scheme array (pScheme) 

if( (styling1 != null) && (styling1[43] != null) )	// IF SC101
{
	if ( (ySeriesSize == 2) && (styling1[1] == "S") )	// IF SC202
	{
		// Use Single Bar Color
		pBarColor = cdvRGBtoColorDefinitionImpl(styling1[43]); // Get bar color
		pScheme[0] = cdvCopyInstance(pBarColor);	// Set single color in 0th entry of schee array
	} // ENDIF-SC202
} // ENDIF-SC101

if (chartMode == "WITHOUTAXES") // IF PIE 209
{
	// Chart WITHOUT AXES
	sd0			= chart.getSeriesDefinitions().get(0); 
	seriesPal2	= sd0.getSeriesPalette( );		// Get Pie Chart palette
} 
else
{
	// Chart WITH AXES
	ySeries0	= yax.getSeriesDefinitions().get(0);
	seriesPal2	= ySeries0.getSeriesPalette(); 
}

seriesPal2.getEntries().clear();
gradDegrees = 0;	// Default

for (ixc=0; ixc < pScheme.length; ixc++)	// IF-PALCAT
{
	if ( (styling1 != null) && (styling1[139]  != null) ) 
	{ 
		color1 = cdvCopyInstance(pScheme[ixc]);
		color2 = cdvCopyInstance(pScheme[ixc]);
		color2 = color2.brighter();
		color2 = color2.brighter();
		
		// Get direction of gradient
		dir = styling1[139].toLowerCase();
		if (dir == "lr") gradDegrees = 0;
		if (dir == "rl") gradDegrees = 0;
		if (dir == "bt") gradDegrees = 90;
		if (dir == "tb") gradDegrees = -90;

		if (dir == "rl") 
		newGrad  = GradientImpl.create(color2, color1, gradDegrees, false);	// Switch colors 1 <-> 2
		else
		newGrad  = GradientImpl.create(color1, color2, gradDegrees, false);
		
		seriesPal2.getEntries().add( newGrad ); 	// Add new gradient to palette
	}
	else
	{ 
		 seriesPal2.getEntries().add( cdvCopyInstance(pScheme[ixc]) ); // Done above
	}  	
} // END IF-PALCAT


if (chartMode != "WITHOUTAXES") // IF 727
{

// FILL THE CATEGORY PALETTE
// =========================
var paletteByCategory = xaxSeries.getSeriesPalette();

paletteByCategory.getEntries().clear();	

for (ixc=0; ixc < pScheme.length; ixc++)	// IF-PALCAT
{
	if ( (styling1 != null) && (styling1[139]  != null) ) 
	{ 
		color1 = cdvCopyInstance(pScheme[ixc]);
		color2 = cdvCopyInstance(pScheme[ixc]);
		color2 = color2.brighter();
		color2 = color2.brighter();
		
		// Get direction of gradient
		dir = styling1[139].toLowerCase();
		if (dir == "lr") gradDegrees = 0;
		if (dir == "rl") gradDegrees = 0;
		if (dir == "bt") gradDegrees = 90;
		if (dir == "tb") gradDegrees = -90;

		if (dir == "rl") 
		newGrad  = GradientImpl.create(color2, color1, gradDegrees, false);	// Switch colors 1 <-> 2
		else
		newGrad  = GradientImpl.create(color1, color2, gradDegrees, false);
		
		paletteByCategory.getEntries().add( newGrad );
	}
	else
	{ 
		paletteByCategory.getEntries().add( cdvCopyInstance(pScheme[ixc]) ); 
	}  	   
} // END IF-PALCAT
} // 727

}  // ========== End cdvBuildChart ============ 



function cdvSetColorArrays()
{

// DEFAULT COLORS

// BJ Orange
bjOrange[0] = ColorDefinitionImpl.create(255,223,171);
bjOrange[1] = ColorDefinitionImpl.create(254,160,0); 
bjOrange[2] = ColorDefinitionImpl.create(246,152,0); 

// BJ Mauve
bjMauve[0] = ColorDefinitionImpl.create(218,200,213 );
bjMauve[1] = ColorDefinitionImpl.create(156,104,141 ); 
bjMauve[2] = ColorDefinitionImpl.create(125,83,113 ); 

// BJ  Heather
bjHeather[0] = ColorDefinitionImpl.create(213,200,226 );
bjHeather[1] = ColorDefinitionImpl.create(149,122,177 ); 
bjHeather[2] = ColorDefinitionImpl.create(99,81,131 ); 

// BJ  Emerald
b0 = ColorDefinitionImpl.create(173,231,206); bjEmerald[0] = b0;
b0 = ColorDefinitionImpl.create(119,201,165); bjEmerald[1] = b0;
b0 = ColorDefinitionImpl.create(50,151,109); bjEmerald[2] = b0;

// BJ  YELLOW
b0 = ColorDefinitionImpl.create(251,236,193); bjYellow[0] = b0;
b0 = ColorDefinitionImpl.create(246,204,122); bjYellow[1] = b0;
b0 = ColorDefinitionImpl.create(229,162,58); bjYellow[2] = b0;

// BJ  Rose
b0 = ColorDefinitionImpl.create(247,201,203); bjRose[0] = b0;
b0 = ColorDefinitionImpl.create(178,77,70); bjRose[1] = b0;
b0 = ColorDefinitionImpl.create(158,62,46); bjRose[2] = b0;

// BJ  Nutmeg
b0 = ColorDefinitionImpl.create(196,158,136); bjNutmeg[0] = b0;
b0 = ColorDefinitionImpl.create(135,97,80); bjNutmeg[1] = b0;
b0 = ColorDefinitionImpl.create(105,80,71); bjNutmeg[2] = b0;

// BJ RED
b0 = ColorDefinitionImpl.create(248,180,191); bjRed[0] = b0;
b0 = ColorDefinitionImpl.create(204,71,72); bjRed[1] = b0;
b0 = ColorDefinitionImpl.create(166,65,73); bjRed[2] = b0;

// BJ  BLUE
b0 = ColorDefinitionImpl.create(198,200,213); bjBlue[0] = b0;
b0 = ColorDefinitionImpl.create(126,130,159); bjBlue[1] = b0;
b0 = ColorDefinitionImpl.create(81,77,92); bjBlue[2] = b0;

// bjPostit
b0 = ColorDefinitionImpl.create(240,216,128); bjPostit[0] = b0;
b0 = ColorDefinitionImpl.create(239,213,117); bjPostit[1] = b0;
b0 = ColorDefinitionImpl.create(239,201,72); bjPostit[2] = b0;

// BJ  CHARCOL
b0 = ColorDefinitionImpl.create(184,185,188); bjCharcol[0] = b0;
b0 = ColorDefinitionImpl.create(169,170,173); bjCharcol[1] = b0;
b0 = ColorDefinitionImpl.create(146,147,150); bjCharcol[2] = b0;

// BJ  TEAL
bjTeal[0] = ColorDefinitionImpl.create(190,220,224);
bjTeal[1] = ColorDefinitionImpl.create(69,132,141);
bjTeal[2] = ColorDefinitionImpl.create(59,114,121);


for (bjRange = 0; bjRange < 3; bjRange++) // Initialize 3 colors palettes: LIGHT,MEDIUM,DARK
{
  
	switch(bjRange)
	{
	case 0:	{ target = bj1; };    break;
	case 1:	{ target = bj2; };    break;
	case 2:	{ target = bj3; };    break;
	default:
	}
	target[0] = bjBlue[bjRange];
	target[1] = bjRed[bjRange];
	target[2] = bjYellow[bjRange];
	target[3] = bjMauve[bjRange];
	target[4] = bjEmerald[bjRange];
	target[5] = bjOrange[bjRange];   
	target[6] = bjNutmeg[bjRange];
	target[7] = bjTeal[bjRange];
	target[8] = bjRose[bjRange];	
	target[9] = bjHeather[bjRange];
	target[10]= bjPostit[bjRange];
	target[11]= bjCharcol[bjRange];
	
} // End for -- 1

}  // End cdvSetColorArrays()


// ===============================================
// function: cdvPaintStripes
// Paint alternate shading Stripes (Marker Ranges) 
// on the back wall of the chart
//============================================== 
//
function cdvPaintStripes(  yax, yScale, pLighterColor , pDarkerColor, chartDim, icsc)
{

var pbScaleMax = yScale.getMax().getValue();  //Returns the value of the 'Max' containment reference.
var pbScaleMin = yScale.getMin().getValue();  //Returns the value of the 'Min' containment reference.
var pbScaleNumberofSteps = yScale.getStepNumber();  //Returns number of steps attribute.
var axisRange = pbScaleMax - pbScaleMin;		// Total range eg: 2,000,000 - 0 = 2,000,000
var stepSize  = axisRange / pbScaleNumberofSteps;  	// eg 2,000,000 / 4 = 500,000 is the marker range

//	
// Loop thru each major tick and create marker range band
//
for (i=0, bandIndic = "LighterColor";  i < pbScaleNumberofSteps; i++)
{
	// Create Start and End values for the Marker Range
	MarkerStart = NumberDataElementImpl.create( pbScaleMin + (stepSize * i) );
	MarkerEnd   = NumberDataElementImpl.create( MarkerStart.getValue() + stepSize );
	
	// Create a marker range at the required place on the Y-AXIS
	var mrBand = MarkerRangeImpl.create( yax, MarkerStart, MarkerEnd, 
	ColorDefinitionImpl.create(0,0,0), 	// Fill Black for now.
	null 								// Outline color
	);	
	
	mrBand.getLabel().setVisible(false);	// Hide the marker range label
	
	// USE BOLDER LINE on BACK WALL STRIPE
	// ===================================
	// Use a slightly heavier line to outline each back wall stipe.
	if (pStripeOutline == "none" )
	{
		mrBand.getOutline().setVisible(false); 
	}
	
	if (pStripeOutline == "normal" )
	{
		mrBand.getOutline().setColor( cdvCopyInstance(pGridLineColor) );
		mrBand.getOutline().setVisible(true); 
		mrBand.getOutline().setThickness(0); // Alter stripe outline thickness here
	}
	
	if (pStripeOutline == "bold" )
	{
		mrBand.getOutline().setColor( cdvCopyInstance(pGridLineColor) );
		mrBand.getOutline().setVisible(true); 
		mrBand.getOutline().setThickness(1); // Alter stripe outline thickness here
	}
	
	
	//
	// IF 2D_DEPTH, do *NOT* outline lowest stripe/marker range.
	// =========================================================
	if(chartDim == "2D_DEPTH")
	{	
		if (i == 0)		// 1st (Lowest) marker range band. No outline! (else it's ugly)
		{	mrBand.getOutline().setVisible(false);  }	// Do not show outline on the lowest range
    } 
    else 
  	{	// All other stripes on 2D_DEPTH bar charts
    	mrBand.getOutline().setVisible(true);	
    }	
    
    if(chartDim == "2D_PLAIN")
    {
    mrBand.getOutline().setColor( cdvCopyInstance(pLineColorM) );
	mrBand.getOutline().setVisible(false); 
	mrBand.getOutline().setThickness(0); 
	}
	
    // ==============================================
    // PAINT the STRIPE (a stripe is a Marker Range)
    // ==============================================
	if (bandIndic == "LighterColor")
	{
		mrBand.setFill( cdvCopyInstance(pLighterColor) );
		bandIndic = "DarkerColor";	// Toggle the band indicator
	}
	else
	{
		mrBand.setFill( cdvCopyInstance(pDarkerColor) );
		bandIndic = "LighterColor";
	}
} // End for loop

} // End of cdvPaintStripes

// =========================================================

function cdvRGBtoColorDefinitionImpl(RGBString)
{

pc = ColorDefinitionImpl.create(0,0,0);  // Default = Black.

var RGBnumber = RGBString + ",";	// Put comma on end so split() will work
var RGBArray = RGBnumber.split(",");

redv	= parseInt(RGBArray[0]);
greenv	= parseInt(RGBArray[1]);
bluev	= parseInt(RGBArray[2]);

if ( (isNaN(redv)) || (isNaN(greenv)) || (isNaN(bluev)) )
{  // Leave deafult as Black
}
else
{

//
// CHECK the RGB numbers are valid.
// Each number must be positive, and between 0 -255.

// 
if (  
(redv >= 0)		&& 
(greenv >= 0) 	&& 
(bluev >= 0)  	&& 
(redv <= 255)	&& 
(greenv <= 255) && 
(bluev <= 255) 	)
{
	pc = ColorDefinitionImpl.create(redv,greenv,bluev);
	
} // END Valid numbers
} // END isNan() tests

return pc;

} // END function


//
// Copy Instance of ColorDefinitionImpl
// 
function cdvCopyInstance(source)
{
var newColorImp = ColorDefinitionImpl.create(
source.getRed(),
source.getGreen(),
source.getBlue(),
source.getTransparency()
); 

return newColorImp;

} // END cdvCopyInstance2


// Copy Instance of ColorDefinitionImpl
// 
function cdvCopyGradientInstance(source)
{
var newGradientImp = GradientImpl.create(
source.getStartColor(),
source.getEndColor(),
source.getDirection(),
false);

return newGradientImp;
} // END cdvCopyGradientInstance()


function cdvSetLegendDefaults(hLegend)
{
// Set LEGEND DEFAULTS
// -------------------

// LEGEND PLACEMENT SETTINGS DEFAULTS
// ----------------------------------
hLegend.setVisible(true);	// Make legend visible

// LEGEND TITLE
hLegend.getTitle().getCaption().getFont().setName("San Serif");
hLegend.getTitle().getCaption().getFont().setSize(20);


// LEGEND ENTRIES
hLegend.getText().getFont().setSize(8); // Font size
hLegend.getText().getFont().setName("San Serif"); // Font face

// BACKGROUND COLOR
hLegend.setBackground( ColorDefinitionImpl.TRANSPARENT() ); 

// POSITION
hLegend.setPosition(Position.BELOW_LITERAL);	// Below the chart
//   Position.ABOVE_LITERAL 
//   Position.BELOW_LITERAL 
//   Position.INSIDE_LITERAL 
//   Position.LEFT_LITERAL 
//   Position.OUTSIDE_LITERAL 
//   Position.RIGHT_LITERAL 

// ANCHOR
hLegend.setAnchor(Anchor.SOUTH_LITERAL );	// TO bottom left
//    Anchor.EAST_LITERAL 
//    Anchor.NORTH_EAST_LITERAL 
//    Anchor.NORTH_LITERAL 
//    Anchor.NORTH_WEST_LITERAL 
//    Anchor.SOUTH_EAST_LITERAL 
//    Anchor.SOUTH_LITERAL 
//    Anchor.SOUTH_WEST_LITERAL 
//    Anchor.WEST_LITERAL 

// ORIENTATION
hLegend.setOrientation(Orientation.HORIZONTAL_LITERAL);	// Align items horizontally 
// hLegend.setOrientation(Orientation.VERTICAL_LITERAL );	// Align items horizontally 

// hLegend.setDirection(Direction.LEFT_RIGHT_LITERAL);	// Align items horizontally 
// Direction.LEFT_RIGHT_LITERAL 
// Direction.TOP_BOTTOM_LITERAL 


// Set spacing around Legend block (aka: insets)
// --------------------------------------------
// Inset parameters:
// 1: ABOVE 
// 2: to the LEFT
// 3: to the BOTTOM 
// 4: to the RIGHT

hLegend.getInsets().setTop(6.0);	// Increase space above legend

// LEGEND OUTLINES
// ---------------
hLegend.getOutline().setVisible(false);	// Hide
hLegend.getSeparator().setVisible(false);
hLegend.getClientArea().getOutline().setVisible(false); 
// Legend Outline
// hLegend.getOutline().setColor( ColorDefinitionImpl.create(255,0,0) );

}  // END cdvSetLegendDefaults(hlegend)

// Function: padLabel: 
// Used when multi-line labels are needed *AND* 
// the centered option is required. 
// Insert leading spaces to center the text.
// Note: Unicode nbsp is used ("\u00A0") else spaces are trimmed out.
// 
function padLabel( icsc, str, width ) 
{
var padding = "";
var slen = str.length(); 

if( slen < width ) 
{
	var padSize = width - str.length();  // eg 8 - 2  = 6 spaces
	padSize = padSize / 2;		// eg: 6/ 2 = 3 pads to to the left
 	padding = repeatChar("\u00A0", padSize); 
    return  padding + str + padding;
}
else
	return  str;
} // End padLabel()


// Function: repeatChar:
// Create an array of character. s=char. n=array length
function repeatChar(s, n){
    var a = [];
    while(a.length < n){ a.push(s); }
    return a.join('');
}

// APPLY INSETS
// ============

function applyInsets(insets, target, icsc)
{
ll_insets = insets + ","	// Put comma on end so split will work
insetsArray = ll_insets.split(',');

// Convert string to numbers
insetsArray[0] = parseInt(insetsArray[0]); 	// top 
insetsArray[1] = parseInt(insetsArray[1]);  // bottom
insetsArray[2] = parseInt(insetsArray[2]);  // left
insetsArray[3] = parseInt(insetsArray[3]);  // right

target.setTop(insetsArray[0]);
target.setBottom(insetsArray[1]); 
target.setLeft(insetsArray[2]);
target.setRight(insetsArray[3]);
}

// APPLY FORMAT SPECIFIER (Assumes NumberFormat)
// =============================================

function applyFormatSpecifier(format, target, icsc)
{
ll_format = format + ","	// Put comma on end so split will work
ll_formatArray = ll_format.split(',');

// ll_formatArray entries are:
// +0 Prefix
// +1 Suffix
// +2 Multiplier
// +3 Number of fraction digits

ll_target = NumberFormatSpecifierImpl.create();

ll_target.setPrefix( ll_formatArray[0].toString());
ll_target.setSuffix( ll_formatArray[1].toString()); 

// If no multiplier defined - set to 1.
if (ll_formatArray[2].length != 0)
ll_target.setMultiplier( parseFloat(ll_formatArray[2]) );
else
ll_target.setMultiplier(1);

// If no fraction digits defined - set to 0.
if (ll_formatArray[3].length != 0)
ll_target.setFractionDigits(parseInt( ll_formatArray[3]) );
else
ll_target.setFractionDigits(0);

// Set the format specifier
target.setFormatSpecifier(ll_target); 

}

function getGradientColor(Color, pBarGradient)
{
c1 = "";
c2 = "";
gradegrees = 0;

if (pBarGradient == "lr") c1 = Color; c2 = Color.brighter().brighter(); gradDegrees = 0;
if (pBarGradient == "rl") c2 = Color; c1 = Color.brighter().brighter(); gradDegrees = 0;
if (pBarGradient == "bt") c1 = Color; c2 = Color.brighter().brighter(); gradDegrees = 90;
if (pBarGradient == "tb") c1 = Color; c2 = Color.brighter().brighter(); gradDegrees = -90;
gradColor  = GradientImpl.create(c1, c2, gradDegrees, false);
return gradColor;

} // End getGradientColor


function cdvCalculateScale2(chart, yscale, icsc, pstyling1)
{

// DEFAULT VALUES
var pMin = 0;
var pMax = 0;
var pStepSize = 0.0;
var pNumberOfSteps = 1;

// GET Y-Axis scale User Overrides - if any
// ========================================

if ( (pstyling1 != null) && (pstyling1[400] != null) )	// s1[400] must be non null
{
//
// Y Axis scale overrides provided.. get them, and use them
//
// s1[401] MIN
// s1[402] MAX
// s1[403] Step Size (Float eg: 250000.0)
// s1[404] Number of steps
// Note: 403 and 404 are mutually exclusive. Only one or the other
// if 403 Step size in defined, 404 is ignored.
// 
if (pstyling1[401] != null) { pMin = parseFloat(pstyling1[401]); } // min
if (pstyling1[402] != null) { pMax = parseFloat(pstyling1[402]); } // max
if (pstyling1[403] != null) { pStepSize = parseFloat(pstyling1[403]); } // step size
if ( (pstyling1[404] != null) && (pstyling1[403] != null)   ) 
	{ pNumberOfSteps = parseInt(pstyling1[404]); } 
}  // End of [400]


// Loop thru all Y-Axis Series to discover the highest value
// =========================================================
var sdefs  	= yax.getSeriesDefinitions().get(0); 
var Yseries	= sdefs.getSeries();
var loopct	= Yseries.size(); 	// Set loop count. eg: 4-1  = 3 series. eg: 2003/4/5
var numInSeries = Yseries[1].getDataSet().getValues().length; // Number of entries in series
var maxValue = 0;				// Initial the max value var.
var newYAxisTop = 0;

if (pMax != 0) {  newYAxisTop = pMax;  }  // User defined max...so use it.

// if (pMax != 0) {  newYAxisTop = pMax; topBarValue = pMax;   rounded = pMax;   }  // User defined max...so use it.

if (pMax == 0)  // IF-NO-USER-DEFINED-MAX
{			

// For each item in a series, sum the total value incase
// we are drawing a Stacked report.
// EG: This values for: Jan 2003 + Jan 2004 + Jan 2005 will be greater than
// one year and the Y-Axis must be adjusted to accommodate the summation of each years totals.
 
var MaxSumArray = new Array();
for (w=0;  w < numInSeries ; w++) // Init array entries to zero. Required.
	{  MaxSumArray[w] = 0; }

// Loop thru All Series
// --------------------
for (index=1;  index < loopct; index++) // outer loop
{
var ySeries			= Yseries[index];
var ySeriesValues	= ySeries.getDataSet().getValues(); 

// Loop thru current Series
	for (q=0 ; q < ySeriesValues.length ; q++) // inner loop
	{ 	
		var theNumber = BirtMath.roundUp(ySeriesValues[q]);
		
		var nn = MaxSumArray[q];
		
		nn = nn + theNumber;
		
		MaxSumArray[q] = nn;
			 
		 // GET HIGHEST INDIVIDUAL VALUE
		if (theNumber > maxValue)
			{ maxValue = theNumber }   // Save new highest value
	}  // End inner for loop
}  // End outer for loop

// Round up the Max value found in the Y-Axis series
// =================================================
maxValue = BirtMath.roundUp(maxValue);	// Round up to a whole number

// DETERMINE WHICH Maximum Value to use.
// ====================================
//
// If there are more than 2 series AND the series are Stacked,
// use the maximum accumulated total.
// eg: Jan 2003 = 33, Jan 2004 = 10, Jan 2005 = 10  : Total = 53
// If the series are stacked, use 53 as the maximum Y-Axis value, not 33. 
// If stacked, the accumulated total must be accommodated on the Y-Axis.
//
if (( ySeriesSize > 2)  &&  (chartSubType == "Stacked"))   // IF-405
{
maxStackedNumber = 0;
for (q=0 ; q < numInSeries ; q++) // 
	{ 	
		// Save new highest value
		if (MaxSumArray[q] > maxStackedNumber) { maxStackedNumber = MaxSumArray[q] } 
	}  
maxValue = BirtMath.roundUp(maxStackedNumber);
} // IF-405

// Calculate the upper value on the Y-AXIS
// ---------------------------------------

// Add 10% to the total. eg: 180,000 + 18,000 = 198,000 (topBarValue)	
var topBarValue = BirtMath.add( BirtMath.divide (maxValue, 10), maxValue  )
	
// Round to a whole number. (dollars only) 
var rounded = BirtMath.round(topBarValue,0)
	
// } // END IF-NO-USER-DEFINED-MAX (So we needed to calcuales the scale)   113

	// Count chars in string. eg 100000 = 6, 10000 = 5
	var charsInValue = rounded.toString().length;
	
switch(charsInValue)
{
	case 2:	yAxis_Adjustment = 10; break;
	case 3:	yAxis_Adjustment = 100; break;
	case 4:	yAxis_Adjustment = 1000; break;
	case 5:	yAxis_Adjustment = 10000; break;
	case 6:	yAxis_Adjustment = 100000; break;
	case 7:	yAxis_Adjustment = 1000000; break;
	case 8:	yAxis_Adjustment = 10000000; break;
	default:
}

// Calculate the new top value on Y-Axis.
// ceiling() will return the new whole above the current value.
// eg: 178,234 will return the next whole 100,000 value which is 200,000

newYAxisTop = BirtMath.ceiling(topBarValue, yAxis_Adjustment);

var yStepSize		= 0;
var yNumberofSteps	= 0;	

// Based on size of max number on y-Axis, calculate the step size.
// eg: Max value = 300, steps are in 100's. 800 steps are in 200's
// ----------------------------------------------------------------

switch(charsInValue)
{
	case 2:	{ if (newYAxisTop < 51) yStepSize = 5; else yStepSize = 10; }; break;
	case 3:	{ if (newYAxisTop < 501) yStepSize = 50; else yStepSize = 100; } break;
	case 4:	{ if (newYAxisTop < 5001) yStepSize = 500; else yStepSize = 1000; } break;
	case 5:	{ if (newYAxisTop < 50001) yStepSize = 5000; else yStepSize = 10000; } break;
	case 6:	{ if (newYAxisTop < 500001) yStepSize = 50000; else yStepSize 	= 100000; } break;
	case 7:	{ if (newYAxisTop < 5000001) yStepSize = 500000; else yStepSize  = 1000000; } break;
	case 8:	{ if (newYAxisTop < 50000001) yStepSize  = 5000000; else yStepSize = 10000000; } break;
	default:
}

}  
 
 
// Calculate number of steps
// =========================
// If user provides step size, use it
if ( (pstyling1 != null) && (pstyling1[400] != null) )	// s1[400] must be non null
{
	if (pstyling1[403] != null) { yStepSize = parseFloat(pstyling1[403]); } // Get step size
}

yNumberofSteps = newYAxisTop  / yStepSize; 

if ( (pstyling1 != null) && (pstyling1[400] != null) )	// s1[400] must be non null
{
	// Use provided number of steps
	if ( (pstyling1[404] != null) && (pstyling1[403] == null)   ) 
	{ 
		yNumberofSteps = parseInt(pstyling1[404]); 
	} 	
}  // End of [400]

// Minimum y-axis number
if (pMin == 0)
	yscale.setMin( NumberDataElementImpl.create(0.0) );
else
	yscale.setMin( NumberDataElementImpl.create(pMin) );	// Use user defined min.

yscale.setMax( NumberDataElementImpl.create(newYAxisTop) );
yscale.setMinorGridsPerUnit(5);
yscale.setStep(1) ;
yscale.setStepNumber(yNumberofSteps);  //Returns number of steps attribute.

}  // End of cdvCalculateScale2()

